# coding: utf-8
from beaker._compat import u_
from beaker.cache import clsmap, Cache, util
from beaker.exceptions import InvalidCacheBackendError
from beaker.middleware import CacheMiddleware
from unittest import SkipTest

try:
    from webtest import TestApp as WebTestApp
except ImportError:
    WebTestApp = None

try:
    clsmap['ext:sqla']._init_dependencies()
except InvalidCacheBackendError:
    raise SkipTest("an appropriate SQLAlchemy backend is not installed")

import sqlalchemy as sa
from beaker.ext.sqla import make_cache_table

engine = sa.create_engine('sqlite://')
metadata = sa.MetaData()
cache_table = make_cache_table(metadata)
metadata.create_all(engine)

def simple_app(environ, start_response):
    extra_args = {}
    clear = False
    if environ.get('beaker.clear'):
        clear = True
    extra_args['type'] = 'ext:sqla'
    extra_args['bind'] = engine
    extra_args['table'] = cache_table
    extra_args['data_dir'] = './cache'
    cache = environ['beaker.cache'].get_cache('testcache', **extra_args)
    if clear:
        cache.clear()
    try:
        value = cache.get_value('value')
    except:
        value = 0
    cache.set_value('value', value+1)
    start_response('200 OK', [('Content-type', 'text/plain')])
    return [('The current value is: %s' % cache.get_value('value')).encode('utf-8')]

def cache_manager_app(environ, start_response):
    cm = environ['beaker.cache']
    cm.get_cache('test')['test_key'] = 'test value'

    start_response('200 OK', [('Content-type', 'text/plain')])
    yield ("test_key is: %s\n" % cm.get_cache('test')['test_key']).encode('utf-8')
    cm.get_cache('test').clear()

    try:
        test_value = cm.get_cache('test')['test_key']
    except KeyError:
        yield ("test_key cleared").encode('utf-8')
    else:
        test_value = cm.get_cache('test')['test_key']
        yield ("test_key wasn't cleared, is: %s\n" % test_value).encode('utf-8')

def make_cache():
    """Return a ``Cache`` for use by the unit tests."""
    return Cache('test', data_dir='./cache', bind=engine, table=cache_table,
                 type='ext:sqla')

def test_has_key():
    cache = make_cache()
    o = object()
    cache.set_value("test", o)
    assert cache.has_key("test")
    assert "test" in cache
    assert not cache.has_key("foo")
    assert "foo" not in cache
    cache.remove_value("test")
    assert not cache.has_key("test")

def test_has_key_multicache():
    cache = make_cache()
    o = object()
    cache.set_value("test", o)
    assert cache.has_key("test")
    assert "test" in cache
    cache = make_cache()
    assert cache.has_key("test")
    cache.remove_value('test')

def test_clear():
    cache = make_cache()
    o = object()
    cache.set_value("test", o)
    assert cache.has_key("test")
    cache.clear()
    assert not cache.has_key("test")

def test_unicode_keys():
    cache = make_cache()
    o = object()
    cache.set_value(u_('hiŏ'), o)
    assert u_('hiŏ') in cache
    assert u_('hŏa') not in cache
    cache.remove_value(u_('hiŏ'))
    assert u_('hiŏ') not in cache

@util.skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_increment():
    app = WebTestApp(CacheMiddleware(simple_app))
    res = app.get('/', extra_environ={'beaker.clear': True})
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res

@util.skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_cache_manager():
    app = WebTestApp(CacheMiddleware(cache_manager_app))
    res = app.get('/')
    assert 'test_key is: test value' in res
    assert 'test_key cleared' in res



# coding: utf-8
from beaker._compat import u_

import unittest.mock

from beaker.cache import Cache, CacheManager, util
from beaker.middleware import CacheMiddleware, SessionMiddleware
from beaker.exceptions import InvalidCacheBackendError
from beaker.util import parse_cache_config_options
import unittest

try:
    from webtest import TestApp as WebTestApp
except ImportError:
    WebTestApp = None

try:
    from beaker.ext import memcached
    client = memcached._load_client()
except InvalidCacheBackendError:
    raise unittest.SkipTest("an appropriate memcached backend is not installed")

mc_url = '127.0.0.1:11211'

c =client.Client([mc_url])
c.set('x', 'y')
if not c.get('x'):
    raise unittest.SkipTest("Memcached is not running at %s" % mc_url)

def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)

def simple_session_app(environ, start_response):
    session = environ['beaker.session']
    sess_id = environ.get('SESSION_ID')
    if environ['PATH_INFO'].startswith('/invalid'):
        # Attempt to access the session
        id = session.id
        session['value'] = 2
    else:
        if sess_id:
            session = session.get_by_id(sess_id)
        if not session:
            start_response('200 OK', [('Content-type', 'text/plain')])
            return ["No session id of %s found." % sess_id]
        if not session.has_key('value'):
            session['value'] = 0
        session['value'] += 1
        if not environ['PATH_INFO'].startswith('/nosave'):
            session.save()
    start_response('200 OK', [('Content-type', 'text/plain')])
    return [
        ('The current value is: %d, session id is %s' % (
            session['value'], session.id
        )).encode('utf-8')
    ]

def simple_app(environ, start_response):
    extra_args = {}
    clear = False
    if environ.get('beaker.clear'):
        clear = True
    extra_args['type'] = 'ext:memcached'
    extra_args['url'] = mc_url
    extra_args['data_dir'] = './cache'
    cache = environ['beaker.cache'].get_cache('testcache', **extra_args)
    if clear:
        cache.clear()
    try:
        value = cache.get_value('value')
    except:
        value = 0
    cache.set_value('value', value+1)
    start_response('200 OK', [('Content-type', 'text/plain')])
    return [
        ('The current value is: %s' % cache.get_value('value')).encode('utf-8')
    ]


def using_none_app(environ, start_response):
    extra_args = {}
    clear = False
    if environ.get('beaker.clear'):
        clear = True
    extra_args['type'] = 'ext:memcached'
    extra_args['url'] = mc_url
    extra_args['data_dir'] = './cache'
    cache = environ['beaker.cache'].get_cache('testcache', **extra_args)
    if clear:
        cache.clear()
    try:
        value = cache.get_value('value')
    except:
        value = 10
    cache.set_value('value', None)
    start_response('200 OK', [('Content-type', 'text/plain')])
    return [
        ('The current value is: %s' % value).encode('utf-8')
    ]


def cache_manager_app(environ, start_response):
    cm = environ['beaker.cache']
    cm.get_cache('test')['test_key'] = 'test value'

    start_response('200 OK', [('Content-type', 'text/plain')])
    yield (
        "test_key is: %s\n" % cm.get_cache('test')['test_key']
    ).encode('utf-8')
    cm.get_cache('test').clear()

    try:
        test_value = cm.get_cache('test')['test_key']
    except KeyError:
        yield "test_key cleared".encode('utf-8')
    else:
        yield ("test_key wasn't cleared, is: %s\n" % (
            cm.get_cache('test')['test_key'],
        )).encode('utf-8')


@util.skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_session():
    app = WebTestApp(SessionMiddleware(simple_session_app, data_dir='./cache', type='ext:memcached', url=mc_url))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res


@util.skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_session_invalid():
    app = WebTestApp(SessionMiddleware(simple_session_app, data_dir='./cache', type='ext:memcached', url=mc_url))
    res = app.get('/invalid', headers=dict(Cookie='beaker.session.id=df7324911e246b70b5781c3c58328442; Path=/'))
    assert 'current value is: 2' in res


def test_has_key():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    o = object()
    cache.set_value("test", o)
    assert cache.has_key("test")
    assert "test" in cache
    assert not cache.has_key("foo")
    assert "foo" not in cache
    cache.remove_value("test")
    assert not cache.has_key("test")

def test_dropping_keys():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    cache.set_value('test', 20)
    cache.set_value('fred', 10)
    assert cache.has_key('test')
    assert 'test' in cache
    assert cache.has_key('fred')

    # Directly nuke the actual key, to simulate it being removed by memcached
    cache.namespace.mc.delete('test_test')
    assert not cache.has_key('test')
    assert cache.has_key('fred')

    # Nuke the keys dict, it might die, who knows
    cache.namespace.mc.delete('test:keys')
    assert cache.has_key('fred')

    # And we still need clear to work, even if it won't work well
    cache.clear()

def test_deleting_keys():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    cache.set_value('test', 20)

    # Nuke the keys dict, it might die, who knows
    cache.namespace.mc.delete('test:keys')

    assert cache.has_key('test')

    # make sure we can still delete keys even though our keys dict got nuked
    del cache['test']

    assert not cache.has_key('test')

def test_has_key_multicache():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    o = object()
    cache.set_value("test", o)
    assert cache.has_key("test")
    assert "test" in cache
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    assert cache.has_key("test")

def test_unicode_keys():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    o = object()
    cache.set_value(u_('hiŏ'), o)
    assert u_('hiŏ') in cache
    assert u_('hŏa') not in cache
    cache.remove_value(u_('hiŏ'))
    assert u_('hiŏ') not in cache

def test_long_unicode_keys():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    o = object()
    long_str = u_('Очень длинная строка, которая не влезает в сто двадцать восемь байт и поэтому не проходит ограничение в check_key, что очень прискорбно, не правда ли, друзья? Давайте же скорее исправим это досадное недоразумение!')
    cache.set_value(long_str, o)
    assert long_str in cache
    cache.remove_value(long_str)
    assert long_str not in cache

def test_spaces_in_unicode_keys():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    o = object()
    cache.set_value(u_('hi ŏ'), o)
    assert u_('hi ŏ') in cache
    assert u_('hŏa') not in cache
    cache.remove_value(u_('hi ŏ'))
    assert u_('hi ŏ') not in cache

def test_spaces_in_keys():
    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
    cache.set_value("has space", 24)
    assert cache.has_key("has space")
    assert 24 == cache.get_value("has space")
    cache.set_value("hasspace", 42)
    assert cache.has_key("hasspace")
    assert 42 == cache.get_value("hasspace")

@util.skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_increment():
    app = WebTestApp(CacheMiddleware(simple_app))
    res = app.get('/', extra_environ={'beaker.clear':True})
    assert 'current value is: 1' in res.text
    res = app.get('/')
    assert 'current value is: 2' in res.text
    res = app.get('/')
    assert 'current value is: 3' in res.text

    app = WebTestApp(CacheMiddleware(simple_app))
    res = app.get('/', extra_environ={'beaker.clear':True})
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res

@util.skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_cache_manager():
    app = WebTestApp(CacheMiddleware(cache_manager_app))
    res = app.get('/')
    assert 'test_key is: test value' in res.text
    assert 'test_key cleared' in res.text

@util.skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_store_none():
    app = WebTestApp(CacheMiddleware(using_none_app))
    res = app.get('/', extra_environ={'beaker.clear':True})
    assert 'current value is: 10' in res.text
    res = app.get('/')
    assert 'current value is: None' in res.text

class TestPylibmcInit(unittest.TestCase):
    def setUp(self):

        from beaker.ext import memcached
        try:
            import pylibmc as memcache
        except:
            import memcache
            memcached._client_libs['pylibmc'] = memcached.pylibmc = memcache
            from contextlib import contextmanager
            class ThreadMappedPool(dict):
                "a mock of pylibmc's ThreadMappedPool"

                def __init__(self, master):
                    self.master = master

                @contextmanager
                def reserve(self):
                    yield self.master
            memcache.ThreadMappedPool = ThreadMappedPool

    def test_uses_pylibmc_client(self):
        from beaker.ext import memcached
        cache = Cache('test', data_dir='./cache',
                            memcache_module='pylibmc',
                            url=mc_url, type="ext:memcached")
        assert isinstance(cache.namespace, memcached.PyLibMCNamespaceManager)

    def test_dont_use_pylibmc_client(self):
        from beaker.ext.memcached import _load_client
        load_mock = unittest.mock.Mock()
        load_mock.return_value = _load_client('memcache')
        with unittest.mock.patch('beaker.ext.memcached._load_client', load_mock):
            cache = Cache('test', data_dir='./cache', url=mc_url, type="ext:memcached")
            assert not isinstance(cache.namespace, memcached.PyLibMCNamespaceManager)
            assert isinstance(cache.namespace, memcached.MemcachedNamespaceManager)

    def test_client(self):
        cache = Cache('test', data_dir='./cache', url=mc_url, type="ext:memcached",
                      protocol='binary')
        o = object()
        cache.set_value("test", o)
        assert cache.has_key("test")
        assert "test" in cache
        assert not cache.has_key("foo")
        assert "foo" not in cache
        cache.remove_value("test")
        assert not cache.has_key("test")

    def test_client_behaviors(self):
        config = {
            'cache.lock_dir':'./lock',
            'cache.data_dir':'./cache',
            'cache.type':'ext:memcached',
            'cache.url':mc_url,
            'cache.memcache_module':'pylibmc',
            'cache.protocol':'binary',
            'cache.behavior.ketama': 'True',
            'cache.behavior.cas':False,
            'cache.behavior.receive_timeout':'3600',
            'cache.behavior.send_timeout':1800,
            'cache.behavior.tcp_nodelay':1,
            'cache.behavior.auto_eject':"0"
        }
        cache_manager = CacheManager(**parse_cache_config_options(config))
        cache = cache_manager.get_cache('test_behavior', expire=6000)

        with cache.namespace.pool.reserve() as mc:
            assert "ketama" in mc.behaviors
            assert mc.behaviors["ketama"] == 1
            assert "cas" in mc.behaviors
            assert mc.behaviors["cas"] == 0
            assert "receive_timeout" in mc.behaviors
            assert mc.behaviors["receive_timeout"] == 3600
            assert "send_timeout" in mc.behaviors
            assert mc.behaviors["send_timeout"] == 1800
            assert "tcp_nodelay" in mc.behaviors
            assert mc.behaviors["tcp_nodelay"] == 1
            assert "auto_eject" in mc.behaviors
            assert mc.behaviors["auto_eject"] == 0

    def test_pylibmc_pool_sharing(self):
        from beaker.ext import memcached
        cache_1a = Cache('test_1a', data_dir='./cache',
                            memcache_module='pylibmc',
                            url=mc_url, type="ext:memcached")
        cache_1b = Cache('test_1b', data_dir='./cache',
                            memcache_module='pylibmc',
                            url=mc_url, type="ext:memcached")
        cache_2 = Cache('test_2', data_dir='./cache',
                            memcache_module='pylibmc',
                            url='127.0.0.1:11212', type="ext:memcached")

        assert (cache_1a.namespace.pool is cache_1b.namespace.pool)
        assert (cache_1a.namespace.pool is not cache_2.namespace.pool)




from beaker.middleware import SessionMiddleware
from beaker.session import Session
import datetime
import re

def test_cookie_expires():
    """Explore valid arguments for cookie_expires."""
    def app(*args, **kw):
        pass

    key = 'beaker.session.cookie_expires'
    now = datetime.datetime.now()

    values = ['300', 300,
        True,  'True',  'true',  't',
        False, 'False', 'false', 'f',
        datetime.timedelta(minutes=5), now]

    expected = [datetime.timedelta(seconds=300),
            datetime.timedelta(seconds=300),
            True, True, True, True,
            False, False, False, False,
            datetime.timedelta(minutes=5), now]

    actual = []

    for pos, v in enumerate(values):
        try:
            s = SessionMiddleware(app, config={key:v})
            val = s.options['cookie_expires']
        except:
            val = None
        assert val == expected[pos]


def cookie_expiration(session):
    cookie = session.cookie.output()
    expiry_m = re.match('Set-Cookie: beaker.session.id=[0-9a-f]{32}(; expires=[^;]+)?; Path=/', cookie)
    assert expiry_m
    expiry = expiry_m.group(1)
    if expiry is None:
        return True
    if re.match('; expires=(Mon|Tue), 1[89]-Jan-2038 [0-9:]{8} GMT', expiry):
        return False
    else:
        return expiry[10:]


def test_cookie_exprires_2():
    """Exhibit Set-Cookie: values."""
    expires = cookie_expiration(Session({}, cookie_expires=True))

    assert expires is True, expires
    no_expires = cookie_expiration(Session({}, cookie_expires=False))

    assert no_expires is False, no_expires

def test_cookie_expires_different_locale():
    from locale import setlocale, LC_TIME
    expires_date = datetime.datetime(2019, 5, 22)
    setlocale(LC_TIME, 'it_IT.UTF-8')
    # if you get locale.Error: unsupported locale setting. you have to enable that locale in your OS.
    assert expires_date.strftime("%a, %d-%b-%Y %H:%M:%S GMT").startswith('mer,')
    session = Session({}, cookie_expires=True, validate_key='validate_key')
    assert session._set_cookie_expires(expires_date)
    expires = cookie_expiration(session)
    assert expires == 'Wed, 22-May-2019 00:00:00 GMT', expires
    setlocale(LC_TIME, '')  # restore default locale for further tests

def test_set_cookie_expires():
    """Exhibit Set-Cookie: values."""
    session = Session({}, cookie_expires=True)
    assert cookie_expiration(session) is True
    session._set_cookie_expires(False)
    assert cookie_expiration(session) is False
    session._set_cookie_expires(True)
    assert cookie_expiration(session) is True



import pytest

from beaker.middleware import SessionMiddleware
from beaker import crypto

webtest = pytest.importorskip("webtest")

pytest.mark.skipif(not crypto.get_crypto_module('default').has_aes,
                   reason="No AES library is installed, can't test " +
                   "cookie-only Sessions")


def simple_app(environ, start_response):
    session = environ['beaker.session']
    if 'value' not in session:
        session['value'] = 0
    session['value'] += 1
    domain = environ.get('domain')
    if domain:
        session.domain = domain
    if not environ['PATH_INFO'].startswith('/nosave'):
        session.save()
    start_response('200 OK', [('Content-type', 'text/plain')])
    msg = 'The current value is: %d and cookie is %s' % (session['value'], session)
    return [msg.encode('utf-8')]


def test_increment():
    options = {'session.validate_key':'hoobermas',
               'session.type':'cookie'}
    app = webtest.TestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res

    res = app.get('/', extra_environ=dict(domain='.hoop.com',
                                          HTTP_HOST='www.hoop.com'))
    assert 'current value is: 1' in res
    assert 'Domain=.hoop.com' in res.headers['Set-Cookie']

    res = app.get('/', extra_environ=dict(HTTP_HOST='www.hoop.com'))
    assert 'Domain=.hoop.com' in res.headers['Set-Cookie']
    assert 'current value is: 2' in res


def test_cookie_attributes_are_preserved():
    options = {'session.type': 'memory',
               'session.httponly': True,
               'session.secure': True,
               'session.cookie_path': '/app',
               'session.cookie_domain': 'localhost'}
    app = webtest.TestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/app', extra_environ=dict(
        HTTP_COOKIE='beaker.session.id=oldsessid', domain='.hoop.com'))
    cookie = res.headers['Set-Cookie']
    assert 'domain=.hoop.com' in cookie.lower()
    assert 'path=/app' in cookie.lower()
    assert 'secure' in cookie.lower()
    assert 'httponly' in cookie.lower()
    assert 'samesite=lax' in cookie.lower()


if __name__ == '__main__':
    from paste import httpserver
    wsgi_app = SessionMiddleware(simple_app, {})
    httpserver.serve(wsgi_app, host='127.0.0.1', port=8080)



from beaker.cache import CacheManager
from beaker.util import parse_cache_config_options
from datetime import datetime

defaults = {'cache.data_dir':'./cache', 'cache.type':'dbm', 'cache.expire': 60, 'cache.regions': 'short_term'}

cache = CacheManager(**parse_cache_config_options(defaults))

def get_cached_value():
    @cache.region('short_term', 'test_namespacing')
    def get_value():
        return datetime.now()

    return get_value()




# -*- coding: utf-8 -*-
from beaker._compat import u_, pickle, b64decode

import binascii
import shutil
import sys
import time
import unittest
import warnings

import pytest

from beaker.container import MemoryNamespaceManager
from beaker.crypto import get_crypto_module
from beaker.exceptions import BeakerException
from beaker.session import CookieSession, Session, SessionObject
from beaker.util import deserialize


def get_session(**kwargs):
    """A shortcut for creating :class:`Session` instance"""
    options = {}
    options.update(**kwargs)
    return Session({}, **options)


COOKIE_REQUEST = {}


def setup_cookie_request():
    COOKIE_REQUEST.clear()


def get_cookie_session(**kwargs):
    """A shortcut for creating :class:`CookieSession` instance"""
    options = {'validate_key': 'test_key'}
    options.update(**kwargs)
    if COOKIE_REQUEST.get('set_cookie'):
        COOKIE_REQUEST['cookie'] = COOKIE_REQUEST.get('cookie_out')
    return CookieSession(COOKIE_REQUEST, **options)


def test_session():
    setup_cookie_request()
    for test_case in (
        check_save_load,
        check_save_load_encryption,
        check_save_load_encryption_cryptography,
        check_decryption_failure,
        check_delete,
        check_revert,
        check_invalidate,
        check_timeout,
    ):
        for session_getter in (get_session, get_cookie_session,):
            setup_cookie_request()
            test_case(session_getter)


def check_save_load(session_getter):
    """Test if the data is actually persistent across requests"""
    session = session_getter()
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.save()

    session = session_getter(id=session.id)
    assert u_('Suomi') in session
    assert u_('Great Britain') in session
    assert u_('Deutchland') in session

    assert session[u_('Suomi')] == u_('Kimi Räikkönen')
    assert session[u_('Great Britain')] == u_('Jenson Button')
    assert session[u_('Deutchland')] == u_('Sebastian Vettel')


@pytest.mark.skipif(not get_crypto_module('default').has_aes)
def check_save_load_encryption(session_getter):
    """Test if the data is actually persistent across requests"""
    session = session_getter(encrypt_key='666a19cf7f61c64c',
                          validate_key='hoobermas')
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.save()

    session = session_getter(id=session.id, encrypt_key='666a19cf7f61c64c',
                          validate_key='hoobermas')
    assert u_('Suomi') in session
    assert u_('Great Britain') in session
    assert u_('Deutchland') in session

    assert session[u_('Suomi')] == u_('Kimi Räikkönen')
    assert session[u_('Great Britain')] == u_('Jenson Button')
    assert session[u_('Deutchland')] == u_('Sebastian Vettel')


# cryptography only works for py3.3+, so skip for python 3.2
@pytest.mark.skipif(sys.version_info[0] == 3 and sys.version_info[1] < 3,
                    reason="Cryptography not supported on Python 3 lower than 3.3")
def check_save_load_encryption_cryptography(session_getter):
    """Test if the data is actually persistent across requests"""
    try:
        get_crypto_module('cryptography').has_aes
    except BeakerException:
        raise unittest.SkipTest()
    session = session_getter(
        encrypt_key='666a19cf7f61c64c',
        validate_key='hoobermas',
        crypto_type='cryptography')
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.save()

    session = session_getter(
        id=session.id, encrypt_key='666a19cf7f61c64c',
        validate_key='hoobermas',
        crypto_type='cryptography')
    assert u_('Suomi') in session
    assert u_('Great Britain') in session
    assert u_('Deutchland') in session

    assert session[u_('Suomi')] == u_('Kimi Räikkönen')
    assert session[u_('Great Britain')] == u_('Jenson Button')
    assert session[u_('Deutchland')] == u_('Sebastian Vettel')


@pytest.mark.skipif(not get_crypto_module('default').has_aes)
def check_decryption_failure(session_getter):
    """Test if the data fails without the right keys"""
    session = session_getter(encrypt_key='666a19cf7f61c64c',
                          validate_key='hoobermas')
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.save()

    session = session_getter(id=session.id, encrypt_key='asfdasdfadsfsadf',
                          validate_key='hoobermas', invalidate_corrupt=True)
    assert u_('Suomi') not in session
    assert u_('Great Britain') not in session


def check_delete(session_getter):
    """Test :meth:`Session.delete`"""
    session = session_getter()
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.delete()

    assert u_('Suomi') not in session
    assert u_('Great Britain') not in session
    assert u_('Deutchland') not in session


def check_revert(session_getter):
    """Test :meth:`Session.revert`"""
    session = session_getter()
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.save()

    session = session_getter(id=session.id)
    del session[u_('Suomi')]
    session[u_('Great Britain')] = u_('Lewis Hamilton')
    session[u_('Deutchland')] = u_('Michael Schumacher')
    session[u_('España')] = u_('Fernando Alonso')
    session.revert()

    assert session[u_('Suomi')] == u_('Kimi Räikkönen')
    assert session[u_('Great Britain')] == u_('Jenson Button')
    assert session[u_('Deutchland')] == u_('Sebastian Vettel')
    assert u_('España') not in session


def check_invalidate(session_getter):
    """Test :meth:`Session.invalidate`"""
    session = session_getter()
    session.save()
    id = session.id
    created = session.created
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.invalidate()
    session.save()

    assert session.id != id
    assert session.created != created
    assert u_('Suomi') not in session
    assert u_('Great Britain') not in session
    assert u_('Deutchland') not in session


def test_regenerate_id():
    """Test :meth:`Session.regenerate_id`"""
    # new session & save
    setup_cookie_request()
    session = get_session()
    orig_id = session.id
    session[u_('foo')] = u_('bar')
    session.save()

    # load session
    session = get_session(id=session.id)
    # data should still be there
    assert session[u_('foo')] == u_('bar')

    # regenerate the id
    session.regenerate_id()

    assert session.id != orig_id

    # data is still there
    assert session[u_('foo')] == u_('bar')

    # should be the new id
    assert 'beaker.session.id=%s' % session.id in session.request['cookie_out']

    # get a new session before calling save
    bunk_sess = get_session(id=session.id)
    assert u_('foo') not in bunk_sess

    # save it
    session.save()

    # make sure we get the data back
    session = get_session(id=session.id)
    assert session[u_('foo')] == u_('bar')


def check_timeout(session_getter):
    """Test if the session times out properly"""
    session = session_getter(timeout=2)
    session.save()
    id = session.id
    created = session.created
    session[u_('Suomi')] = u_('Kimi Räikkönen')
    session[u_('Great Britain')] = u_('Jenson Button')
    session[u_('Deutchland')] = u_('Sebastian Vettel')
    session.save()

    session = session_getter(id=session.id, timeout=2)
    assert session.id == id
    assert session.created == created
    assert session[u_('Suomi')] == u_('Kimi Räikkönen')
    assert session[u_('Great Britain')] == u_('Jenson Button')
    assert session[u_('Deutchland')] == u_('Sebastian Vettel')

    time.sleep(2)
    session = session_getter(id=session.id, timeout=2)
    assert session.id != id
    assert session.created != created
    assert u_('Suomi') not in session
    assert u_('Great Britain') not in session
    assert u_('Deutchland') not in session


def test_timeout_requires_accessed_time():
    """Test that it doesn't allow setting save_accessed_time to False with
    timeout enabled
    """
    setup_cookie_request()
    get_session(timeout=None, save_accessed_time=True)  # is ok
    get_session(timeout=None, save_accessed_time=False)  # is ok
    with pytest.raises(BeakerException):
        get_session(timeout=2, save_accessed_time=False)


def test_cookies_enabled():
    """
    Test if cookies are sent out properly when ``use_cookies``
    is set to ``True``
    """
    setup_cookie_request()
    session = get_session(use_cookies=True)
    assert 'cookie_out' in session.request
    assert not session.request['set_cookie']

    session.domain = 'example.com'
    session.path = '/example'
    assert session.request['set_cookie']
    assert 'beaker.session.id=%s' % session.id in session.request['cookie_out']
    assert 'Domain=example.com' in session.request['cookie_out']
    assert 'Path=/' in session.request['cookie_out']

    session = get_session(use_cookies=True)
    session.save()
    assert session.request['set_cookie']
    assert 'beaker.session.id=%s' % session.id in session.request['cookie_out']

    session = get_session(use_cookies=True, id=session.id)
    session.delete()
    assert session.request['set_cookie']
    assert 'beaker.session.id=%s' % session.id in session.request['cookie_out']
    assert 'expires=' in session.request['cookie_out']

    # test for secure
    session = get_session(use_cookies=True, secure=True)
    cookie = session.request['cookie_out'].lower()  # Python3.4.3 outputs "Secure", while previous output "secure"
    assert 'secure' in cookie, cookie

    # test for httponly
    class ShowWarning(object):
        def __init__(self):
            self.msg = None

        def __call__(self, message, category, filename, lineno, file=None, line=None):
            self.msg = str(message)
    orig_sw = warnings.showwarning
    sw = ShowWarning()
    warnings.showwarning = sw
    session = get_session(use_cookies=True, httponly=True)
    if sys.version_info < (2, 6):
        assert sw.msg == 'Python 2.6+ is required to use httponly'
    else:
        # Python3.4.3 outputs "HttpOnly", while previous output "httponly"
        cookie = session.request['cookie_out'].lower()
        assert 'httponly' in cookie, cookie
    warnings.showwarning = orig_sw


def test_cookies_disabled():
    """
    Test that no cookies are sent when ``use_cookies`` is set to ``False``
    """
    session = get_session(use_cookies=False)
    assert 'set_cookie' not in session.request
    assert 'cookie_out' not in session.request

    session.save()
    assert 'set_cookie' not in session.request
    assert 'cookie_out' not in session.request

    session = get_session(use_cookies=False, id=session.id)
    assert 'set_cookie' not in session.request
    assert 'cookie_out' not in session.request

    session.delete()
    assert 'set_cookie' not in session.request
    assert 'cookie_out' not in session.request


def test_file_based_replace_optimization():
    """Test the file-based backend with session,
    which includes the 'replace' optimization.

    """
    setup_cookie_request()

    session = get_session(use_cookies=False, type='file',
                            data_dir='./cache')

    session['foo'] = 'foo'
    session['bar'] = 'bar'
    session.save()

    session = get_session(use_cookies=False, type='file',
                            data_dir='./cache', id=session.id)
    assert session['foo'] == 'foo'
    assert session['bar'] == 'bar'

    session['bar'] = 'bat'
    session['bat'] = 'hoho'
    session.save()

    session.namespace.do_open('c', False)
    session.namespace['test'] = 'some test'
    session.namespace.do_close()

    session = get_session(use_cookies=False, type='file',
                            data_dir='./cache', id=session.id)

    session.namespace.do_open('r', False)
    assert session.namespace['test'] == 'some test'
    session.namespace.do_close()

    assert session['foo'] == 'foo'
    assert session['bar'] == 'bat'
    assert session['bat'] == 'hoho'
    session.save()

    # the file has been replaced, so our out-of-session
    # key is gone
    session.namespace.do_open('r', False)
    assert 'test' not in session.namespace
    session.namespace.do_close()

def test_use_json_serializer_without_encryption_key():
    setup_cookie_request()
    so = get_session(use_cookies=False, type='file', data_dir='./cache', data_serializer='json')
    so['foo'] = 'bar'
    so.save()
    session = get_session(id=so.id, use_cookies=False, type='file', data_dir='./cache', data_serializer='json')
    assert 'foo' in session
    serialized_session = open(session.namespace.file, 'rb').read()
    memory_state = pickle.loads(serialized_session)
    session_data = b64decode(memory_state.get('session'))
    data = deserialize(session_data, 'json')
    assert 'foo' in data


def test_invalidate_corrupt():
    setup_cookie_request()
    session = get_session(use_cookies=False, type='file',
                          data_dir='./cache')
    session['foo'] = 'bar'
    session.save()

    f = open(session.namespace.file, 'w')
    f.write("crap")
    f.close()

    with pytest.raises((pickle.UnpicklingError, EOFError, TypeError, binascii.Error,)):
        get_session(use_cookies=False, type='file',
                    data_dir='./cache', id=session.id)

    session = get_session(use_cookies=False, type='file',
                            invalidate_corrupt=True,
                            data_dir='./cache', id=session.id)
    assert "foo" not in dict(session)


def test_invalidate_empty_cookie():
    setup_cookie_request()
    kwargs = {'validate_key': 'test_key', 'encrypt_key': 'encrypt'}
    session = get_cookie_session(**kwargs)
    session['foo'] = 'bar'
    session.save()

    COOKIE_REQUEST['cookie_out'] = ' beaker.session.id='
    session = get_cookie_session(id=session.id, invalidate_corrupt=False, **kwargs)
    assert "foo" not in dict(session)


def test_unrelated_cookie():
    setup_cookie_request()
    kwargs = {'validate_key': 'test_key', 'encrypt_key': 'encrypt'}
    session = get_cookie_session(**kwargs)
    session['foo'] = 'bar'
    session.save()

    COOKIE_REQUEST['cookie_out'] = COOKIE_REQUEST['cookie_out'] + '; some.other=cookie'
    session = get_cookie_session(id=session.id, invalidate_corrupt=False, **kwargs)
    assert "foo" in dict(session)


def test_invalidate_invalid_signed_cookie():
    setup_cookie_request()
    kwargs = {'validate_key': 'test_key', 'encrypt_key': 'encrypt'}
    session = get_cookie_session(**kwargs)
    session['foo'] = 'bar'
    session.save()

    COOKIE_REQUEST['cookie_out'] = (
        COOKIE_REQUEST['cookie_out'][:20] +
        'aaaaa' +
        COOKIE_REQUEST['cookie_out'][25:]
    )

    with pytest.raises(BeakerException):
        get_cookie_session(id=session.id, invalidate_corrupt=False)


def test_invalidate_invalid_signed_cookie_invalidate_corrupt():
    setup_cookie_request()
    kwargs = {'validate_key': 'test_key', 'encrypt_key': 'encrypt'}
    session = get_cookie_session(**kwargs)
    session['foo'] = 'bar'
    session.save()

    COOKIE_REQUEST['cookie_out'] = (
        COOKIE_REQUEST['cookie_out'][:20] +
        'aaaaa' +
        COOKIE_REQUEST['cookie_out'][25:]
    )

    session = get_cookie_session(id=session.id, invalidate_corrupt=True, **kwargs)
    assert "foo" not in dict(session)


def test_load_deleted_from_storage_session__not_loaded():
    req = {'cookie': {'beaker.session.id': 123}}
    session = Session(req, timeout=1)

    session.delete()
    session.save()

    Session(req, timeout=1)


class TestSaveAccessedTime(unittest.TestCase):
    # These tests can't use the memory session type since it seems that loading
    # winds up with references to the underlying storage and makes changes to
    # sessions even though they aren't save()ed.
    def setUp(self):
        # Ignore errors because in most cases the dir won't exist.
        shutil.rmtree('./cache', ignore_errors=True)

    def tearDown(self):
        shutil.rmtree('./cache')

    def test_saves_if_session_written_and_accessed_time_false(self):
        session = get_session(data_dir='./cache', save_accessed_time=False)
        # New sessions are treated a little differently so save the session
        # before getting into the meat of the test.
        session.save()
        session = get_session(data_dir='./cache', save_accessed_time=False,
                              id=session.id)
        last_accessed = session.last_accessed
        session.save(accessed_only=False)
        session = get_session(data_dir='./cache', save_accessed_time=False,
                              id=session.id)
        # If the second save saved, we'll have a new last_accessed time.
        # Python 2.6 doesn't have assertGreater :-(
        assert session.last_accessed > last_accessed, (
            '%r is not greater than %r' %
            (session.last_accessed, last_accessed))

    def test_saves_if_session_not_written_and_accessed_time_true(self):
        session = get_session(data_dir='./cache', save_accessed_time=True)
        # New sessions are treated a little differently so save the session
        # before getting into the meat of the test.
        session.save()
        session = get_session(data_dir='./cache', save_accessed_time=True,
                              id=session.id)
        last_accessed = session.last_accessed
        session.save(accessed_only=True)  # this is the save we're really testing
        session = get_session(data_dir='./cache', save_accessed_time=True,
                              id=session.id)
        # If the second save saved, we'll have a new last_accessed time.
        # Python 2.6 doesn't have assertGreater :-(
        assert session.last_accessed > last_accessed, (
            '%r is not greater than %r' %
            (session.last_accessed, last_accessed))

    def test_doesnt_save_if_session_not_written_and_accessed_time_false(self):
        session = get_session(data_dir='./cache', save_accessed_time=False)
        # New sessions are treated a little differently so save the session
        # before getting into the meat of the test.
        session.save()
        session = get_session(data_dir='./cache', save_accessed_time=False,
                              id=session.id)
        last_accessed = session.last_accessed
        session.save(accessed_only=True)  # this shouldn't actually save
        session = get_session(data_dir='./cache', save_accessed_time=False,
                              id=session.id)
        self.assertEqual(session.last_accessed, last_accessed)


class TestSessionObject(unittest.TestCase):
    def setUp(self):
        # San check that we are in fact using the memory backend...
        assert get_session().namespace_class == MemoryNamespaceManager
        # so we can be sure we're clearing the right state.
        MemoryNamespaceManager.namespaces.clear()

    def test_no_autosave_saves_atime_without_save(self):
        so = SessionObject({}, auto=False)
        so['foo'] = 'bar'
        so.persist()
        session = get_session(id=so.id)
        assert '_accessed_time' in session
        assert 'foo' not in session  # because we didn't save()

    def test_no_autosave_saves_with_save(self):
        so = SessionObject({}, auto=False)
        so['foo'] = 'bar'
        so.save()
        so.persist()
        session = get_session(id=so.id)
        assert '_accessed_time' in session
        assert 'foo' in session

    def test_no_autosave_saves_with_delete(self):
        req = {'cookie': {'beaker.session.id': 123}}

        so = SessionObject(req, auto=False)
        so['foo'] = 'bar'
        so.save()
        so.persist()
        session = get_session(id=so.id)
        assert 'foo' in session

        so2 = SessionObject(req, auto=False)
        so2.delete()
        so2.persist()
        session = get_session(id=so2.id)
        assert 'foo' not in session

    def test_auto_save_saves_without_save(self):
        so = SessionObject({}, auto=True)
        so['foo'] = 'bar'
        # look ma, no save()!
        so.persist()
        session = get_session(id=so.id)
        assert 'foo' in session

    def test_accessed_time_off_saves_atime_when_saving(self):
        so = SessionObject({}, save_accessed_time=False)
        atime = so['_accessed_time']
        so['foo'] = 'bar'
        so.save()
        so.persist()
        session = get_session(id=so.id, save_accessed_time=False)
        assert 'foo' in session
        assert '_accessed_time' in session
        self.assertEqual(session.last_accessed, atime)

    def test_accessed_time_off_doesnt_save_without_save(self):
        req = {'cookie': {'beaker.session.id': 123}}
        so = SessionObject(req, save_accessed_time=False)
        so.persist()  # so we can do a set on a non-new session

        so2 = SessionObject(req, save_accessed_time=False)
        so2['foo'] = 'bar'
        # no save()
        so2.persist()

        session = get_session(id=so.id, save_accessed_time=False)
        assert 'foo' not in session



from beaker._compat import u_
import unittest

from beaker.converters import asbool, aslist


class AsBool(unittest.TestCase):
    def test_truth_str(self):
        for v in ('true', 'yes', 'on', 'y', 't', '1'):
            self.assertTrue(asbool(v), "%s should be considered True" % (v,))
            v = v.upper()
            self.assertTrue(asbool(v), "%s should be considered True" % (v,))

    def test_false_str(self):
        for v in ('false', 'no', 'off', 'n', 'f', '0'):
            self.assertFalse(asbool(v), v)
            v = v.upper()
            self.assertFalse(asbool(v), v)

    def test_coerce(self):
        """Things that can coerce right straight to booleans."""
        self.assertTrue(asbool(True))
        self.assertTrue(asbool(1))
        self.assertTrue(asbool(42))
        self.assertFalse(asbool(False))
        self.assertFalse(asbool(0))

    def test_bad_values(self):
        self.assertRaises(ValueError, asbool, ('mommy!'))
        self.assertRaises(ValueError, asbool, (u_('Blargl?')))


class AsList(unittest.TestCase):
    def test_string(self):
        self.assertEqual(aslist('abc'), ['abc'])
        self.assertEqual(aslist('1a2a3', 'a'), ['1', '2', '3'])

    def test_None(self):
        self.assertEqual(aslist(None), [])

    def test_listy_noops(self):
        """Lists and tuples should come back unchanged."""
        x = [1, 2, 3]
        self.assertEqual(aslist(x), x)
        y = ('z', 'y', 'x')
        self.assertEqual(aslist(y), y)

    def test_listify(self):
        """Other objects should just result in a single item list."""
        self.assertEqual(aslist(dict()), [{}])


if __name__ == '__main__':
    unittest.main()




import time
from datetime import datetime

import shutil

from beaker.cache import CacheManager, cache_regions
from beaker.util import parse_cache_config_options

defaults = {'cache.data_dir':'./cache', 'cache.type':'dbm', 'cache.expire': 2}

def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)

def make_cache_obj(**kwargs):
    opts = defaults.copy()
    opts.update(kwargs)
    cache = CacheManager(**parse_cache_config_options(opts))
    return cache

def make_region_cached_func():
    global _cache_obj
    opts = {}
    opts['cache.regions'] = 'short_term, long_term'
    opts['cache.short_term.expire'] = '2'
    cache = make_cache_obj(**opts)

    @cache.region('short_term', 'region_loader')
    def load(person):
        now = datetime.now()
        return "Hi there %s, its currently %s" % (person, now)
    _cache_obj = cache
    return load

def make_cached_func():
    global _cache_obj
    cache = make_cache_obj()

    @cache.cache('loader')
    def load(person):
        now = datetime.now()
        return "Hi there %s, its currently %s" % (person, now)
    _cache_obj = cache
    return load

def test_parse_doesnt_allow_none():
    opts = {}
    opts['cache.regions'] = 'short_term, long_term'
    for region, params in parse_cache_config_options(opts)['cache_regions'].items():
        for k, v in params.items():
            assert v != 'None', k

def test_parse_doesnt_allow_empty_region_name():
    opts = {}
    opts['cache.regions'] = ''
    regions = parse_cache_config_options(opts)['cache_regions']
    assert len(regions) == 0

def test_decorators():
    for func in (make_region_cached_func, make_cached_func):
        check_decorator(func())

def check_decorator(func):
    result = func('Fred')
    assert 'Fred' in result

    result2 = func('Fred')
    assert result == result2

    result3 = func('George')
    assert 'George' in result3
    result4 = func('George')
    assert result3 == result4

    time.sleep(2)
    result2 = func('Fred')
    assert result != result2

def test_check_invalidate_region():
    func = make_region_cached_func()
    result = func('Fred')
    assert 'Fred' in result

    result2 = func('Fred')
    assert result == result2
    _cache_obj.region_invalidate(func, None, 'region_loader', 'Fred')

    result3 = func('Fred')
    assert result3 != result2

    result2 = func('Fred')
    assert result3 == result2

    # Invalidate a non-existent key
    _cache_obj.region_invalidate(func, None, 'region_loader', 'Fredd')
    assert result3 == result2

def test_check_invalidate():
    func = make_cached_func()
    result = func('Fred')
    assert 'Fred' in result

    result2 = func('Fred')
    assert result == result2
    _cache_obj.invalidate(func, 'loader', 'Fred')

    result3 = func('Fred')
    assert result3 != result2

    result2 = func('Fred')
    assert result3 == result2

    # Invalidate a non-existent key
    _cache_obj.invalidate(func, 'loader', 'Fredd')
    assert result3 == result2

def test_long_name():
    func = make_cached_func()
    name = 'Fred' * 250
    result = func(name)
    assert name in result

    result2 = func(name)
    assert result == result2
    # This won't actually invalidate it since the key won't be sha'd
    _cache_obj.invalidate(func, 'loader', name, key_length=8000)

    result3 = func(name)
    assert result3 == result2

    # And now this should invalidate it
    _cache_obj.invalidate(func, 'loader', name)
    result4 = func(name)
    assert result3 != result4


def test_cache_region_has_default_key_length():
    try:
        cache = CacheManager(cache_regions={
            'short_term_without_key_length':{
                'expire': 60,
                'type': 'memory'
            }
        })

        # Check CacheManager registered the region in global regions
        assert 'short_term_without_key_length' in cache_regions

        @cache.region('short_term_without_key_length')
        def load_without_key_length(person):
            now = datetime.now()
            return "Hi there %s, its currently %s" % (person, now)

        # Ensure that same person gets same time
        msg = load_without_key_length('fred')
        msg2 = load_without_key_length('fred')
        assert msg == msg2, (msg, msg2)

        # Ensure that different person gets different time
        msg3 = load_without_key_length('george')
        assert msg3.split(',')[-1] != msg2.split(',')[-1]

    finally:
        # throw away region for this test
        cache_regions.pop('short_term_without_key_length', None)


def test_cache_region_expire_is_always_int():
    try:
        cache = CacheManager(cache_regions={
            'short_term_with_string_expire': {
                'expire': '60',
                'type': 'memory'
            }
        })

        # Check CacheManager registered the region in global regions
        assert 'short_term_with_string_expire' in cache_regions

        @cache.region('short_term_with_string_expire')
        def load_with_str_expire(person):
            now = datetime.now()
            return "Hi there %s, its currently %s" % (person, now)

        # Ensure that same person gets same time
        msg = load_with_str_expire('fred')
        msg2 = load_with_str_expire('fred')
        assert msg == msg2, (msg, msg2)

    finally:
        # throw away region for this test
        cache_regions.pop('short_term_with_string_expire', None)


def test_directory_goes_away():
    cache = CacheManager(cache_regions={
        'short_term_without_key_length':{
            'expire': 60,
            'type': 'dbm',
            'data_dir': '/tmp/beaker-tests/cache/data',
            'lock_dir': '/tmp/beaker-tests/cache/lock'
        }
    })


    @cache.region('short_term_without_key_length')
    def load_with_str_expire(person):
        now = datetime.now()
        return "Hi there %s, its currently %s" % (person, now)


    # Ensure that same person gets same time
    msg = load_with_str_expire('fred')
    msg2 = load_with_str_expire('fred')

    shutil.rmtree('/tmp/beaker-tests')

    msg3 = load_with_str_expire('fred')
    assert msg == msg2, (msg, msg2)
    assert msg2 != msg3, (msg2, msg3)



# coding: utf-8
from beaker._compat import u_, bytes_

import os
import platform
import shutil
import tarfile
import tempfile
import time
from beaker.middleware import CacheMiddleware
from beaker import util
from beaker.cache import Cache
from unittest import SkipTest
from beaker.util import skip_if
import base64
import zlib

try:
    from webtest import TestApp as WebTestApp
except ImportError:
    WebTestApp = None

# Tarballs of the output of:
# >>> from beaker.cache import Cache
# >>> c = Cache('test', data_dir='db', type='dbm')
# >>> c['foo'] = 'bar'
# in the old format, Beaker @ revision: 24f57102d310
dbm_cache_tar = bytes_("""\
eJzt3EtOwkAAgOEBjTHEBDfu2ekKZ6bTTnsBL+ABzPRB4osSRBMXHsNruXDl3nMYLaEbpYRAaIn6
f8kwhFcn/APLSeNTUTdZsL4/m4Pg21wSqiCt9D1PC6mUZ7Xo+bWvrHB/N3HjXk+MrrLhQ/a48HXL
nv+l0vg0yYcTdznMxhdpfFvHbpj1lyv0N8oq+jdhrr/b/A5Yo79R9G9ERX8XbXgLrNHfav7/G1Hd
30XGhYPMT5JYRbELVGISGVov9SKVRaGNQj2I49TrF+8oxpJrTAMHxizob+b7ay+Y/v5lE1/AP+8v
9o5ccdsWYvdViMPpIwdCtMRsiP3yTrucd8r5pJxbz8On9/KT2uVo3H5rG1cFAAAAAOD3aIuP7lv3
pRjbXgkAAAAAAFjVyc1Idc6U1lYGgbSmL0Mjpe248+PYjY87I91x/UGeb3udAAAAAACgfh+fAAAA
AADgr/t5/sPFTZ5cb/38D19Lzn9pRHX/zR4CtEZ/o+nfiEX9N3kI0Gr9vWl/W0z0BwAAAAAAAAAA
AAAAAAAAqPAFyOvcKA==
""")
dbm_cache_tar = zlib.decompress(base64.b64decode(dbm_cache_tar))

# dumbdbm format
dumbdbm_cache_tar = bytes_("""\
eJzt191qgzAYBmCPvYqc2UGx+ZKY6A3scCe7gJKoha6binOD3f2yn5Ouf3TTlNH3AQlEJcE3nyGV
W0RT457Jsq9W6632W0Se0JI49/1E0vCIZZPPzHt5HmzPWNQ91M1r/XbwuVP3/6nKLcq2Gey6qftl
5Z6mWA3n56/IKOQfwk7+dvwV8Iv8FSH/IPbkb4uRl8BZ+fvg/WUE8g9if/62UDZf1VlZOiqc1VSq
kudGVrKgushNkYuVc5VM/Rups5vjY3wErJU6nD+Z7fyFNFpEjIf4AFeef7Jq22TOZnzOpLiJLz0d
CGyE+q/scHyMk/Wv+E79G0L9hzC7JSFMpv0PN0+J4rv7xNk+iTuKh07E6aXnB9Mao/7X/fExzt//
FecS9R8C9v/r9rP+l49tubnk+e/z/J8JjvMfAAAAAAAAAADAn70DFJAAwQ==
""")
dumbdbm_cache_tar = zlib.decompress(base64.b64decode(dumbdbm_cache_tar))

def simple_app(environ, start_response):
    clear = False
    if environ.get('beaker.clear'):
        clear = True
    cache = environ['beaker.cache'].get_cache('testcache')
    if clear:
        cache.clear()
    try:
        value = cache.get_value('value')
    except:
        value = 0
    cache.set_value('value', value+1)
    start_response('200 OK', [('Content-type', 'text/plain')])
    msg = 'The current value is: %s' % cache.get_value('value')
    return [msg.encode('utf-8')]

def cache_manager_app(environ, start_response):
    cm = environ['beaker.cache']
    cm.get_cache('test')['test_key'] = 'test value'

    start_response('200 OK', [('Content-type', 'text/plain')])
    yield ("test_key is: %s\n" % cm.get_cache('test')['test_key']).encode('utf-8')
    cm.get_cache('test').clear()

    try:
        test_value = cm.get_cache('test')['test_key']
    except KeyError:
        yield "test_key cleared".encode('utf-8')
    else:
        test_value = cm.get_cache('test')['test_key']
        yield ("test_key wasn't cleared, is: %s\n" % test_value).encode('utf-8')

def test_has_key():
    cache = Cache('test', data_dir='./cache', type='dbm')
    o = object()
    cache.set_value("test", o)
    assert cache.has_key("test")
    assert "test" in cache
    assert not cache.has_key("foo")
    assert "foo" not in cache
    cache.remove_value("test")
    assert not cache.has_key("test")

def test_expire_changes():
    cache = Cache('test_bar', data_dir='./cache', type='dbm')
    cache.set_value('test', 10)
    assert cache.has_key('test')
    assert cache['test'] == 10

    # ensure that we can change a never-expiring value
    cache.set_value('test', 20, expiretime=1)
    assert cache.has_key('test')
    assert cache['test'] == 20
    time.sleep(1)
    assert not cache.has_key('test')

    # test that we can change it before its expired
    cache.set_value('test', 30, expiretime=50)
    assert cache.has_key('test')
    assert cache['test'] == 30

    cache.set_value('test', 40, expiretime=3)
    assert cache.has_key('test')
    assert cache['test'] == 40
    time.sleep(3)
    assert not cache.has_key('test')

def test_fresh_createfunc():
    cache = Cache('test_foo', data_dir='./cache', type='dbm')
    x = cache.get_value('test', createfunc=lambda: 10, expiretime=2)
    assert x == 10
    x = cache.get_value('test', createfunc=lambda: 12, expiretime=2)
    assert x == 10
    x = cache.get_value('test', createfunc=lambda: 14, expiretime=2)
    assert x == 10
    time.sleep(2)
    x = cache.get_value('test', createfunc=lambda: 16, expiretime=2)
    assert x == 16
    x = cache.get_value('test', createfunc=lambda: 18, expiretime=2)
    assert x == 16

    cache.remove_value('test')
    assert not cache.has_key('test')
    x = cache.get_value('test', createfunc=lambda: 20, expiretime=2)
    assert x == 20

def test_has_key_multicache():
    cache = Cache('test', data_dir='./cache', type='dbm')
    o = object()
    cache.set_value("test", o)
    assert cache.has_key("test")
    assert "test" in cache
    cache = Cache('test', data_dir='./cache', type='dbm')
    assert cache.has_key("test")

def test_unicode_keys():
    cache = Cache('test', data_dir='./cache', type='dbm')
    o = object()
    cache.set_value(u_('hiŏ'), o)
    assert u_('hiŏ') in cache
    assert u_('hŏa') not in cache
    cache.remove_value(u_('hiŏ'))
    assert u_('hiŏ') not in cache

def test_remove_stale():
    """test that remove_value() removes even if the value is expired."""

    cache = Cache('test', type='memory')
    o = object()
    cache.namespace[b'key'] = (time.time() - 60, 5, o)
    container = cache._get_value('key')
    assert not container.has_current_value()
    assert b'key' in container.namespace
    cache.remove_value('key')
    assert b'key' not in container.namespace

    # safe to call again
    cache.remove_value('key')

def test_multi_keys():
    cache = Cache('newtests', data_dir='./cache', type='dbm')
    cache.clear()
    called = {}
    def create_func():
        called['here'] = True
        return 'howdy'

    try:
        cache.get_value('key1')
    except KeyError:
        pass
    else:
        raise Exception("Failed to keyerror on nonexistent key")

    assert 'howdy' == cache.get_value('key2', createfunc=create_func)
    assert called['here'] == True
    del called['here']

    try:
        cache.get_value('key3')
    except KeyError:
        pass
    else:
        raise Exception("Failed to keyerror on nonexistent key")
    try:
        cache.get_value('key1')
    except KeyError:
        pass
    else:
        raise Exception("Failed to keyerror on nonexistent key")

    assert 'howdy' == cache.get_value('key2', createfunc=create_func)
    assert called == {}

@skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_increment():
    app = WebTestApp(CacheMiddleware(simple_app))
    res = app.get('/', extra_environ={'beaker.type':type, 'beaker.clear':True})
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res

@skip_if(lambda: WebTestApp is None, "webtest not installed")
def test_cache_manager():
    app = WebTestApp(CacheMiddleware(cache_manager_app))
    res = app.get('/')
    assert 'test_key is: test value' in res
    assert 'test_key cleared' in res

def test_clsmap_nonexistent():
    from beaker.cache import clsmap

    try:
        clsmap['fake']
        assert False
    except KeyError:
        pass

def test_clsmap_present():
    from beaker.cache import clsmap

    assert clsmap['memory']


def test_legacy_cache():
    cache = Cache('newtests', data_dir='./cache', type='dbm')

    cache.set_value('x', '1')
    assert cache.get_value('x') == '1'

    cache.set_value('x', '2', type='file', data_dir='./cache')
    assert cache.get_value('x') == '1'
    assert cache.get_value('x', type='file', data_dir='./cache') == '2'

    cache.remove_value('x')
    cache.remove_value('x', type='file', data_dir='./cache')

    assert cache.get_value('x', expiretime=1, createfunc=lambda: '5') == '5'
    assert cache.get_value('x', expiretime=1, createfunc=lambda: '6', type='file', data_dir='./cache') == '6'
    assert cache.get_value('x', expiretime=1, createfunc=lambda: '7') == '5'
    assert cache.get_value('x', expiretime=1, createfunc=lambda: '8', type='file', data_dir='./cache') == '6'
    time.sleep(1)
    assert cache.get_value('x', expiretime=1, createfunc=lambda: '9') == '9'
    assert cache.get_value('x', expiretime=1, createfunc=lambda: '10', type='file', data_dir='./cache') == '10'
    assert cache.get_value('x', expiretime=1, createfunc=lambda: '11') == '9'
    assert cache.get_value('x', expiretime=1, createfunc=lambda: '12', type='file', data_dir='./cache') == '10'


def test_upgrade():
    # If we're on OSX, lets run this since its OSX dump files, otherwise
    # we have to skip it
    if platform.system() != 'Darwin':
        return
    for test in _test_upgrade_has_key, _test_upgrade_in, _test_upgrade_setitem:
        for mod, tar in (('dbm', dbm_cache_tar),
                         ('dumbdbm', dumbdbm_cache_tar)):
            try:
                __import__(mod)
            except ImportError:
                continue
            dir = tempfile.mkdtemp()
            fd, name = tempfile.mkstemp(dir=dir)
            fp = os.fdopen(fd, 'wb')
            fp.write(tar)
            fp.close()
            tar = tarfile.open(name)
            for member in tar.getmembers():
                tar.extract(member, dir)
            tar.close()
            try:
                test(os.path.join(dir, 'db'))
            finally:
                shutil.rmtree(dir)

def _test_upgrade_has_key(dir):
    cache = Cache('test', data_dir=dir, type='dbm')
    assert cache.has_key('foo')
    assert cache.has_key('foo')

def _test_upgrade_in(dir):
    cache = Cache('test', data_dir=dir, type='dbm')
    assert 'foo' in cache
    assert 'foo' in cache

def _test_upgrade_setitem(dir):
    cache = Cache('test', data_dir=dir, type='dbm')
    assert cache['foo'] == 'bar'
    assert cache['foo'] == 'bar'


def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)



from beaker.cache import Cache
from . import base


class TestRedis(base.CacheManagerBaseTests):
    CACHE_ARGS = {
        'type': 'ext:redis',
        'url': 'redis://localhost:6379/13'
    }

    def test_client_reuse(self):
        cache1 = Cache('test1', **self.CACHE_ARGS)
        cli1 = cache1.namespace.client
        cache2 = Cache('test2', **self.CACHE_ARGS)
        cli2 = cache2.namespace.client
        self.assertTrue(cli1 is cli2)


from __future__ import print_function
import time


def go():
    from . import namespace_get
    a = namespace_get.get_cached_value()
    time.sleep(0.3)
    b = namespace_get.get_cached_value()

    time.sleep(0.3)

    from ..test_namespacing_files import namespace_get as upper_ns_get
    c = upper_ns_get.get_cached_value()
    time.sleep(0.3)
    d = upper_ns_get.get_cached_value()

    print(a)
    print(b)
    print(c)
    print(d)

    assert a == b, 'Basic caching problem - should never happen'
    assert c == d, 'Basic caching problem - should never happen'
    assert a == c, 'Namespaces not consistent when using different import paths'



from __future__ import unicode_literals

from binascii import b2a_hex, a2b_hex
from beaker.crypto.pbkdf2 import pbkdf2


def test_pbkdf2_test1():
    result = pbkdf2("password", "ATHENA.MIT.EDUraeburn", 1, dklen=16)
    expected = a2b_hex(b"cdedb5281bb2f801565a1122b2563515")
    assert result == expected, (result, expected)


def test_pbkdf2_test2():
    result = b2a_hex(pbkdf2("password", "ATHENA.MIT.EDUraeburn", 1200, dklen=32))
    expected = b"5c08eb61fdf71e4e4ec3cf6ba1f5512ba7e52ddbc5e5142f708a31e2e62b1e13"
    assert result == expected, (result, expected)


def test_pbkdf2_test3():
    result = b2a_hex(pbkdf2("X"*64, "pass phrase equals block size", 1200, dklen=32))
    expected = b"139c30c0966bc32ba55fdbf212530ac9c5ec59f1a452f5cc9ad940fea0598ed1"
    assert result == expected, (result, expected)


def test_pbkdf2_test4():
    result = b2a_hex(pbkdf2("X"*65, "pass phrase exceeds block size", 1200, dklen=32))
    expected = b"9ccad6d468770cd51b10e6a68721be611a8b4d282601db3b36be9246915ec82a"
    assert result == expected, (result, expected)


def test_pbkd2_issue81():
    """Test for Regression on Incorrect behavior of bytes_() under Python3.4

    https://github.com/bbangert/beaker/issues/81
    """
    result = pbkdf2("MASTER_KEY", b"SALT", 1)
    expected = pbkdf2("MASTER_KEY", "SALT", 1)
    assert result == expected, (result, expected)



import datetime
import time
import re
import json

import beaker.session
import beaker.util
from beaker.session import SignedCookie
from beaker._compat import b64decode
from beaker.middleware import SessionMiddleware
from unittest import SkipTest
try:
    from webtest import TestApp as WebTestApp
except ImportError:
    raise SkipTest("webtest not installed")

from beaker import crypto
if not crypto.get_crypto_module('default').has_aes:
    raise SkipTest("No AES library is installed, can't test cookie-only "
                   "Sessions")

def simple_app(environ, start_response):
    session = environ['beaker.session']
    if 'value' not in session:
        session['value'] = 0
    session['value'] += 1
    if not environ['PATH_INFO'].startswith('/nosave'):
        session.save()
    start_response('200 OK', [('Content-type', 'text/plain')])
    msg = 'The current value is: %d and cookie is %s' % (session['value'], session)
    return [msg.encode('UTF-8')]

def test_increment():
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res

def test_invalid_cookie():
    # This is not actually a cookie only session, but we still test the cookie part.
    options = {'session.validate_key':'hoobermas'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))

    res = app.get('/')
    assert 'current value is: 1' in res

    # Set an invalid cookie.
    app.set_cookie('cb_/zabbix/actionconf.php_parts', 'HI')
    res = app.get('/')
    assert 'current value is: 2' in res, res

    res = app.get('/')
    assert 'current value is: 3' in res, res

def test_invalid_cookie_cookietype():
    # This is not actually a cookie only session, but we still test the cookie part.
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))

    res = app.get('/')
    assert 'current value is: 1' in res

    # Set an invalid cookie.
    app.set_cookie('cb_/zabbix/actionconf.php_parts', 'HI')
    res = app.get('/')
    assert 'current value is: 2' in res, res

    res = app.get('/')
    assert 'current value is: 3' in res, res

def test_json_serializer():
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie', 'data_serializer': 'json'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))

    res = app.get('/')
    assert 'current value is: 1' in res

    res = app.get('/')
    cookie = SignedCookie('hoobermas')
    session_data = cookie.value_decode(app.cookies['beaker.session.id'])[0]
    session_data = b64decode(session_data)
    data = beaker.util.deserialize(session_data, 'json')
    assert data['value'] == 2

    res = app.get('/')
    assert 'current value is: 3' in res

def test_pickle_serializer():
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie', 'data_serializer': 'pickle'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))

    res = app.get('/')
    assert 'current value is: 1' in res

    res = app.get('/')
    cookie = SignedCookie('hoobermas')
    session_data = cookie.value_decode(app.cookies['beaker.session.id'])[0]
    session_data = b64decode(session_data)
    data = beaker.util.deserialize(session_data, 'pickle')
    assert data['value'] == 2

    res = app.get('/')
    assert 'current value is: 3' in res

def test_custom_serializer():
    was_used = [False, False]
    class CustomSerializer(object):
        def loads(self, data_string):
            was_used[0] = True
            return json.loads(data_string.decode('utf-8'))

        def dumps(self, data):
            was_used[1] = True
            return json.dumps(data).encode('utf-8')

    serializer = CustomSerializer()
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie', 'data_serializer': serializer}
    app = WebTestApp(SessionMiddleware(simple_app, **options))

    res = app.get('/')
    assert 'current value is: 1' in res

    res = app.get('/')
    cookie = SignedCookie('hoobermas')
    session_data = cookie.value_decode(app.cookies['beaker.session.id'])[0]
    session_data = b64decode(session_data)
    data = serializer.loads(session_data)
    assert data['value'] == 2

    res = app.get('/')
    assert 'current value is: 3' in res

    assert all(was_used)

def test_expires():
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie',
               'session.cookie_expires': datetime.timedelta(days=1)}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'expires=' in res.headers.getall('Set-Cookie')[0]
    assert 'current value is: 1' in res

def test_different_sessions():
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    app2 = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    res = app2.get('/')
    res = app2.get('/')
    res2 = app.get('/')
    assert 'current value is: 2' in res2
    assert 'current value is: 4' in res

def test_nosave():
    options = {'session.validate_key':'hoobermas', 'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/nosave')
    assert 'current value is: 1' in res
    assert [] == res.headers.getall('Set-Cookie')
    res = app.get('/nosave')
    assert 'current value is: 1' in res

    res = app.get('/')
    assert 'current value is: 1' in res
    assert len(res.headers.getall('Set-Cookie')) > 0
    res = app.get('/')
    assert 'current value is: 2' in res

def test_increment_with_encryption():
    options = {'session.encrypt_key':'666a19cf7f61c64c', 'session.validate_key':'hoobermas',
               'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res

def test_different_sessions_with_encryption():
    options = {'session.encrypt_key':'666a19cf7f61c64c', 'session.validate_key':'hoobermas',
               'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    app2 = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    res = app2.get('/')
    res = app2.get('/')
    res2 = app.get('/')
    assert 'current value is: 2' in res2
    assert 'current value is: 4' in res

def test_nosave_with_encryption():
    options = {'session.encrypt_key':'666a19cf7f61c64c', 'session.validate_key':'hoobermas',
               'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/nosave')
    assert 'current value is: 1' in res
    assert [] == res.headers.getall('Set-Cookie')
    res = app.get('/nosave')
    assert 'current value is: 1' in res

    res = app.get('/')
    assert 'current value is: 1' in res
    assert len(res.headers.getall('Set-Cookie')) > 0
    res = app.get('/')
    assert 'current value is: 2' in res

def test_cookie_id():
    options = {'session.encrypt_key':'666a19cf7f61c64c', 'session.validate_key':'hoobermas',
               'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))

    res = app.get('/')
    assert "_id':" in res
    sess_id = re.sub(r".*'_id': '(.*?)'.*", r'\1', res.body.decode('utf-8'))
    res = app.get('/')
    new_id = re.sub(r".*'_id': '(.*?)'.*", r'\1', res.body.decode('utf-8'))
    assert new_id == sess_id

def test_invalidate_with_save_does_not_delete_session():
    def invalidate_session_app(environ, start_response):
        session = environ['beaker.session']
        session.invalidate()
        session.save()
        start_response('200 OK', [('Content-type', 'text/plain')])
        return [('Cookie is %s' % session).encode('UTF-8')]

    options = {'session.encrypt_key':'666a19cf7f61c64c', 'session.validate_key':'hoobermas',
               'session.type':'cookie'}
    app = WebTestApp(SessionMiddleware(invalidate_session_app, **options))
    res = app.get('/')
    assert 'expires=' not in res.headers.getall('Set-Cookie')[0]


def test_changing_encrypt_key_with_timeout():
    COMMON_ENCRYPT_KEY = '666a19cf7f61c64c'
    DIFFERENT_ENCRYPT_KEY = 'hello-world'

    options = {'session.encrypt_key': COMMON_ENCRYPT_KEY,
               'session.timeout': 300,
               'session.validate_key': 'hoobermas',
               'session.type': 'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'The current value is: 1' in res, res

    # Get the session cookie, so we can reuse it.
    cookies = res.headers['Set-Cookie']

    # Check that we get the same session with the same cookie
    options = {'session.encrypt_key': COMMON_ENCRYPT_KEY,
               'session.timeout': 300,
               'session.validate_key': 'hoobermas',
               'session.type': 'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/', headers={'Cookie': cookies})
    assert 'The current value is: 2' in res, res

    # Now that we are sure that it reuses the same session,
    # change the encrypt_key so that it is unable to understand the cookie.
    options = {'session.encrypt_key': DIFFERENT_ENCRYPT_KEY,
               'session.timeout': 300,
               'session.validate_key': 'hoobermas',
               'session.type': 'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/', headers={'Cookie': cookies})

    # Let's check it created a new session as the old one is invalid
    # in the past it just crashed.
    assert 'The current value is: 1' in res, res


def test_cookie_properly_expires():
    COMMON_ENCRYPT_KEY = '666a19cf7f61c64c'

    options = {'session.encrypt_key': COMMON_ENCRYPT_KEY,
               'session.timeout': 1,
               'session.validate_key': 'hoobermas',
               'session.type': 'cookie'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'The current value is: 1' in res, res

    res = app.get('/')
    assert 'The current value is: 2' in res, res

    # Wait session to expire and check it starts with a clean one
    time.sleep(1)
    res = app.get('/')
    assert 'The current value is: 1' in res, res


def test_cookie_attributes_are_preserved():
    options = {'session.type': 'cookie',
               'session.validate_key': 'hoobermas',
               'session.httponly': True,
               'session.secure': True,
               'session.samesite': 'Strict'}
    app = WebTestApp(SessionMiddleware(simple_app, options))
    res = app.get('/')
    cookie = res.headers['Set-Cookie']
    assert 'secure' in cookie.lower()
    assert 'httponly' in cookie.lower()
    assert 'samesite=strict' in cookie.lower()


def test_cookie_path_properly_set_after_init():
    COOKIE_PATH = '/app'

    options = {
        'session.validate_key': 'hoobermas',
        'session.type': 'cookie',
        'session.cookie_path': COOKIE_PATH,
    }
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/app')
    cookie = res.headers['Set-Cookie']

    assert ('path=%s' % COOKIE_PATH) in cookie.lower()


def test_cookie_path_properly_set_after_load():
    COOKIE_PATH = '/app'

    options = {
        'session.validate_key': 'hoobermas',
        'session.type': 'cookie',
        'session.cookie_path': COOKIE_PATH,
    }
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    # Perform one request to set the cookie
    res = app.get('/app')
    # Perform another request to load the previous session from the cookie
    res = app.get('/app')
    cookie = res.headers['Set-Cookie']

    assert ('path=%s' % COOKIE_PATH) in cookie.lower()


def test_cookie_path_properly_set_after_delete():
    COOKIE_PATH = '/app'

    def delete_session_app(environ, start_response):
        session = environ['beaker.session']
        session.delete()
        start_response('200 OK', [('Content-type', 'text/plain')])
        return [('Cookie is %s' % session).encode('UTF-8')]

    options = {
        'session.validate_key': 'hoobermas',
        'session.type': 'cookie',
        'session.cookie_path': COOKIE_PATH,
    }
    app = WebTestApp(SessionMiddleware(delete_session_app, **options))
    res = app.get('/app')
    cookie = res.headers['Set-Cookie']

    assert ('path=%s' % COOKIE_PATH) in cookie.lower()


if __name__ == '__main__':
    from paste import httpserver
    wsgi_app = SessionMiddleware(simple_app, {})
    httpserver.serve(wsgi_app, host='127.0.0.1', port=8080)



import os
import sys


def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)


def test_consistent_namespacing():
    sys.path.append(os.path.dirname(__file__))
    from tests.test_namespacing_files.namespace_go import go
    go()



import time
from datetime import datetime

from beaker.cache import CacheManager, cache_region, region_invalidate
from beaker import util
from unittest import SkipTest

defaults = {'cache.data_dir':'./cache', 'cache.type':'dbm', 'cache.expire': 2}

def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)

@cache_region('short_term')
def fred(x):
    return time.time()

@cache_region('short_term')
def george(x):
    return time.time()

@cache_region('short_term')
def albert(x):
    """A doc string"""
    return time.time()

@cache_region('short_term')
def alfred(x, xx, y=None):
    return str(time.time()) + str(x) + str(xx) + str(y)

class AlfredCacher(object):
    @cache_region('short_term')
    def alfred_self(self, xx, y=None):
        return str(time.time()) + str(self) + str(xx) + str(y)

try:
    from .annotated_functions import AnnotatedAlfredCacher
except (ImportError, SyntaxError):
    AnnotatedAlfredCacher = None


def make_cache_obj(**kwargs):
    opts = defaults.copy()
    opts.update(kwargs)
    cache = CacheManager(**util.parse_cache_config_options(opts))
    return cache

def make_cached_func(**opts):
    cache = make_cache_obj(**opts)
    @cache.cache()
    def load(person):
        now = datetime.now()
        return "Hi there %s, its currently %s" % (person, now)
    return cache, load

def make_region_cached_func():
    opts = {}
    opts['cache.regions'] = 'short_term, long_term'
    opts['cache.short_term.expire'] = '2'
    cache = make_cache_obj(**opts)

    @cache_region('short_term', 'region_loader')
    def load(person):
        now = datetime.now()
        return "Hi there %s, its currently %s" % (person, now)
    return load

def make_region_cached_func_2():
    opts = {}
    opts['cache.regions'] = 'short_term, long_term'
    opts['cache.short_term.expire'] = '2'
    cache = make_cache_obj(**opts)

    @cache_region('short_term')
    def load_person(person):
        now = datetime.now()
        return "Hi there %s, its currently %s" % (person, now)
    return load_person

def test_check_region_decorator():
    func = make_region_cached_func()
    result = func('Fred')
    assert 'Fred' in result

    result2 = func('Fred')
    assert result == result2

    result3 = func('George')
    assert 'George' in result3
    result4 = func('George')
    assert result3 == result4

    time.sleep(2)  # Now it should have expired as cache is 2secs
    result2 = func('Fred')
    assert result != result2

def test_different_default_names():
    result = fred(1)
    time.sleep(0.1)
    result2 = george(1)
    assert result != result2

def test_check_invalidate_region():
    func = make_region_cached_func()
    result = func('Fred')
    assert 'Fred' in result

    result2 = func('Fred')
    assert result == result2
    region_invalidate(func, None, 'region_loader', 'Fred')

    result3 = func('Fred')
    assert result3 != result2

    result2 = func('Fred')
    assert result3 == result2

    # Invalidate a non-existent key
    region_invalidate(func, None, 'region_loader', 'Fredd')
    assert result3 == result2


def test_check_invalidate_region_2():
    func = make_region_cached_func_2()
    result = func('Fred')
    assert 'Fred' in result

    result2 = func('Fred')
    assert result == result2
    region_invalidate(func, None, 'Fred')

    result3 = func('Fred')
    assert result3 != result2

    result2 = func('Fred')
    assert result3 == result2

    # Invalidate a non-existent key
    region_invalidate(func, None, 'Fredd')
    assert result3 == result2

def test_invalidate_cache():
    cache, func = make_cached_func()
    val = func('foo')
    time.sleep(0.1)
    val2 = func('foo')
    assert val == val2

    cache.invalidate(func, 'foo')
    val3 = func('foo')
    assert val3 != val

def test_class_key_cache():
    cache = make_cache_obj()

    class Foo(object):
        @cache.cache('method')
        def go(self, x, y):
            return "hi foo"

    @cache.cache('standalone')
    def go(x, y):
        return "hi standalone"

    x = Foo().go(1, 2)
    y = go(1, 2)

    ns = go._arg_namespace
    assert cache.get_cache(ns).get('method 1 2') == x
    assert cache.get_cache(ns).get('standalone 1 2') == y

def test_func_namespace():
    def go(x, y):
        return "hi standalone"

    assert 'test_cache_decorator' in util.func_namespace(go)
    assert util.func_namespace(go).endswith('go')

def test_class_key_region():
    opts = {}
    opts['cache.regions'] = 'short_term'
    opts['cache.short_term.expire'] = '2'
    cache = make_cache_obj(**opts)

    class Foo(object):
        @cache_region('short_term', 'method')
        def go(self, x, y):
            return "hi foo"

    @cache_region('short_term', 'standalone')
    def go(x, y):
        return "hi standalone"

    x = Foo().go(1, 2)
    y = go(1, 2)
    ns = go._arg_namespace
    assert cache.get_cache_region(ns, 'short_term').get('method 1 2') == x
    assert cache.get_cache_region(ns, 'short_term').get('standalone 1 2') == y

def test_classmethod_key_region():
    opts = {}
    opts['cache.regions'] = 'short_term'
    opts['cache.short_term.expire'] = '2'
    cache = make_cache_obj(**opts)

    class Foo(object):
        @classmethod
        @cache_region('short_term', 'method')
        def go(cls, x, y):
            return "hi"

    x = Foo.go(1, 2)
    ns = Foo.go._arg_namespace
    assert cache.get_cache_region(ns, 'short_term').get('method 1 2') == x

def test_class_key_region_invalidate():
    opts = {}
    opts['cache.regions'] = 'short_term'
    opts['cache.short_term.expire'] = '2'
    cache = make_cache_obj(**opts)

    class Foo(object):
        @cache_region('short_term', 'method')
        def go(self, x, y):
            now = datetime.now()
            return "hi %s" % now

        def invalidate(self, x, y):
            region_invalidate(self.go, None, "method", x, y)

    x = Foo().go(1, 2)
    time.sleep(0.1)
    y = Foo().go(1, 2)
    Foo().invalidate(1, 2)
    z = Foo().go(1, 2)

    assert x == y
    assert x != z

def test_check_region_decorator_keeps_docstring_and_name():
    result = albert(1)
    time.sleep(0.1)
    result2 = albert(1)
    assert result == result2

    assert albert.__doc__ == "A doc string"
    assert albert.__name__ == "albert"


def test_check_region_decorator_with_kwargs():
    result = alfred(1, xx=5, y=3)
    time.sleep(0.1)

    result2 = alfred(1, y=3, xx=5)
    assert result == result2

    result3 = alfred(1, 5, y=5)
    assert result != result3

    result4 = alfred(1, 5, 3)
    assert result == result4

    result5 = alfred(1, 5, y=3)
    assert result == result5


def test_check_region_decorator_with_kwargs_and_self():
    a1 = AlfredCacher()
    a2 = AlfredCacher()

    result = a1.alfred_self(xx=5, y='blah')
    time.sleep(0.1)

    result2 = a2.alfred_self(y='blah', xx=5)
    assert result == result2

    result3 = a2.alfred_self(5, y=5)
    assert result != result3

    result4 = a2.alfred_self(5, 'blah')
    assert result == result4

    result5 = a2.alfred_self(5, y='blah')
    assert result == result5

    result6 = a2.alfred_self(6, 'blah')
    assert result != result6


def test_check_region_decorator_with_kwargs_self_and_annotations():
    if AnnotatedAlfredCacher is None:
        raise SkipTest('Python version not supporting annotations')

    a1 = AnnotatedAlfredCacher()
    a2 = AnnotatedAlfredCacher()

    result = a1.alfred_self(xx=5, y='blah')
    time.sleep(0.1)

    result2 = a2.alfred_self(y='blah', xx=5)
    assert result == result2

    result3 = a2.alfred_self(5, y=5)
    assert result != result3

    result4 = a2.alfred_self(5, 'blah')
    assert result == result4

    result5 = a2.alfred_self(5, y='blah')
    assert result == result5

    result6 = a2.alfred_self(6, 'blah')
    assert result != result6



from beaker.synchronization import *

# TODO: spawn threads, test locking.


def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)

def test_reentrant_file():
    sync1 = file_synchronizer('test', lock_dir='./cache')
    sync2 = file_synchronizer('test', lock_dir='./cache')
    sync1.acquire_write_lock()
    sync2.acquire_write_lock()
    sync2.release_write_lock()
    sync1.release_write_lock()

def test_null():
    sync = null_synchronizer()
    assert sync.acquire_write_lock()
    sync.release_write_lock()

def test_mutex():
    sync = mutex_synchronizer('someident')
    sync.acquire_write_lock()
    sync.release_write_lock()




import os
import pickle
import random
import shutil
import sys
import time
import pytest

from beaker.container import *
from beaker.synchronization import _synchronizers
from beaker.cache import clsmap
from threading import Thread

class CachedWidget(object):
    totalcreates = 0
    delay = 0

    def __init__(self):
        CachedWidget.totalcreates += 1
        time.sleep(CachedWidget.delay)
        self.time = time.time()

def _run_container_test(cls, totaltime, expiretime, delay, threadlocal):
    print("\ntesting %s for %d secs with expiretime %s delay %d" % (
        cls, totaltime, expiretime, delay))

    CachedWidget.totalcreates = 0
    CachedWidget.delay = delay

    # allow for python overhead when checking current time against expire times
    fudge = 10

    starttime = time.time()

    running = [True]
    class RunThread(Thread):
        def run(self):
            print("%s starting" % self)

            if threadlocal:
                localvalue = Value(
                                'test',
                                cls('test', data_dir='./cache'),
                                createfunc=CachedWidget,
                                expiretime=expiretime,
                                starttime=starttime)
                localvalue.clear_value()
            else:
                localvalue = value

            try:
                while running[0]:
                    item = localvalue.get_value()
                    if expiretime is not None:
                        currenttime = time.time()
                        itemtime = item.time
                        assert itemtime + expiretime + delay + fudge >= currenttime, \
                            "created: %f expire: %f delay: %f currenttime: %f" % \
                            (itemtime, expiretime, delay, currenttime)
                    time.sleep(random.random() * .00001)
            except:
                running[0] = False
                raise
            print("%s finishing" % self)

    if not threadlocal:
        value = Value(
                    'test',
                    cls('test', data_dir='./cache'),
                    createfunc=CachedWidget,
                    expiretime=expiretime,
                    starttime=starttime)
        value.clear_value()
    else:
        value = None

    threads = [RunThread() for i in range(1, 8)]

    for t in threads:
        t.start()

    time.sleep(totaltime)

    failed = not running[0]
    running[0] = False

    for t in threads:
        t.join()

    assert not failed, "One or more threads failed"
    if expiretime is None:
        expected = 1
    else:
        expected = totaltime / expiretime + 1
    assert CachedWidget.totalcreates <= expected, \
            "Number of creates %d exceeds expected max %d" % (CachedWidget.totalcreates, expected)

def test_memory_container(totaltime=10, expiretime=None, delay=0, threadlocal=False):
    _run_container_test(clsmap['memory'],
                  totaltime, expiretime, delay, threadlocal)

def test_dbm_container(totaltime=10, expiretime=None, delay=0):
    _run_container_test(clsmap['dbm'], totaltime, expiretime, delay, False)

def test_file_container(totaltime=10, expiretime=None, delay=0, threadlocal=False):
    _run_container_test(clsmap['file'], totaltime, expiretime, delay, threadlocal)

def test_memory_container_tlocal():
    test_memory_container(expiretime=15, delay=2, threadlocal=True)

def test_memory_container_2():
    test_memory_container(expiretime=12)

def test_memory_container_3():
    test_memory_container(expiretime=15, delay=2)

def test_dbm_container_2():
    test_dbm_container(expiretime=12)

def test_dbm_container_3():
    test_dbm_container(expiretime=15, delay=2)

def test_file_container_2():
    test_file_container(expiretime=12)

def test_file_container_3():
    test_file_container(expiretime=15, delay=2)

def test_file_container_tlocal():
    test_file_container(expiretime=15, delay=2, threadlocal=True)


@pytest.mark.skipif(sys.version_info < (3, 6),
                    reason="Cryptography not supported on Python 3 lower than 3.6")
def test_file_open_bug():
    """ensure errors raised during reads or writes don't lock the namespace open."""

    value = Value('test', clsmap['file']('reentrant_test', data_dir='./cache'))
    if os.path.exists(value.namespace.file):
        os.remove(value.namespace.file)

    value.set_value("x")

    f = open(value.namespace.file, 'w')
    f.write("BLAH BLAH BLAH")
    f.close()

    with pytest.raises(pickle.UnpicklingError):
        value.set_value("y")

    _synchronizers.clear()

    value = Value('test', clsmap['file']('reentrant_test', data_dir='./cache'))

    # TODO: do we have an assertRaises() in nose to use here ?
    with pytest.raises(pickle.UnpicklingError):
        value.set_value("z")


def test_removing_file_refreshes():
    """test that the cache doesn't ignore file removals"""

    x = [0]

    def create():
        x[0] += 1
        return x[0]

    value = Value('test',
                    clsmap['file']('refresh_test', data_dir='./cache'),
                    createfunc=create, starttime=time.time()
                    )
    if os.path.exists(value.namespace.file):
        os.remove(value.namespace.file)
    assert value.get_value() == 1
    assert value.get_value() == 1
    os.remove(value.namespace.file)
    assert value.get_value() == 2


def teardown_module():
    shutil.rmtree('./cache', True)



from beaker.util import SyncDict, WeakValuedRegistry
import random, time, weakref
import threading

class Value(object):
    values = {}

    def do_something(self, id):
        Value.values[id] = self

    def stop_doing_something(self, id):
        del Value.values[id]

mutex = threading.Lock()

def create(id):
    assert not Value.values, "values still remain"
    global totalcreates
    totalcreates += 1
    return Value()

def threadtest(s, id):
    print("create thread %d starting" % id)

    global running
    global totalgets
    while running:
        try:
            value = s.get('test', lambda: create(id))
            value.do_something(id)
        except Exception as e:
            print("Error", e)
            running = False
            break
        else:
            totalgets += 1
            time.sleep(random.random() * .01)
            value.stop_doing_something(id)
            del value
            time.sleep(random.random() * .01)

def runtest(s):

    global values
    values = {}

    global totalcreates
    totalcreates = 0

    global totalgets
    totalgets = 0

    global running
    running = True

    threads = []
    for id_ in range(1, 20):
        t = threading.Thread(target=threadtest, args=(s, id_))
        t.start()
        threads.append(t)

    for i in range(0, 10):
        if not running:
            break
        time.sleep(1)

    failed = not running

    running = False

    for t in threads:
        t.join()

    assert not failed, "test failed"

    print("total object creates %d" % totalcreates)
    print("total object gets %d" % totalgets)


def test_dict():
    # normal dictionary test, where we will remove the value
    # periodically. the number of creates should be equal to
    # the number of removes plus one.
    print("\ntesting with normal dict")
    runtest(SyncDict())


def test_weakdict():
    print("\ntesting with weak dict")
    runtest(WeakValuedRegistry())



# coding: utf-8
import threading
import unittest

import time

import datetime

from beaker._compat import u_
from beaker.cache import Cache
from beaker.middleware import SessionMiddleware, CacheMiddleware
from webtest import TestApp as WebTestApp


class CacheManagerBaseTests(unittest.TestCase):
    SUPPORTS_EXPIRATION = True
    SUPPORTS_TIMEOUT = True
    CACHE_ARGS = {}

    @classmethod
    def setUpClass(cls):
        def simple_session_app(environ, start_response):
            session = environ['beaker.session']
            sess_id = environ.get('SESSION_ID')
            if environ['PATH_INFO'].startswith('/invalid'):
                # Attempt to access the session
                id = session.id
                session['value'] = 2
            else:
                if sess_id:
                    session = session.get_by_id(sess_id)
                if not session:
                    start_response('200 OK', [('Content-type', 'text/plain')])
                    return [("No session id of %s found." % sess_id).encode('utf-8')]
                if not session.has_key('value'):
                    session['value'] = 0
                session['value'] += 1
                if not environ['PATH_INFO'].startswith('/nosave'):
                    session.save()
            start_response('200 OK', [('Content-type', 'text/plain')])
            return [('The current value is: %d, session id is %s' % (session['value'],
                                                                     session.id)).encode('utf-8')]

        def simple_app(environ, start_response):
            extra_args = cls.CACHE_ARGS
            clear = False
            if environ.get('beaker.clear'):
                clear = True
            cache = environ['beaker.cache'].get_cache('testcache', **extra_args)
            if clear:
                cache.clear()
            try:
                value = cache.get_value('value')
            except:
                value = 0
            cache.set_value('value', value + 1)
            start_response('200 OK', [('Content-type', 'text/plain')])
            return [('The current value is: %s' % cache.get_value('value')).encode('utf-8')]

        def using_none_app(environ, start_response):
            extra_args = cls.CACHE_ARGS
            clear = False
            if environ.get('beaker.clear'):
                clear = True
            cache = environ['beaker.cache'].get_cache('testcache', **extra_args)
            if clear:
                cache.clear()
            try:
                value = cache.get_value('value')
            except:
                value = 10
            cache.set_value('value', None)
            start_response('200 OK', [('Content-type', 'text/plain')])
            return [('The current value is: %s' % value).encode('utf-8')]

        def cache_manager_app(environ, start_response):
            cm = environ['beaker.cache']
            cm.get_cache('test')['test_key'] = 'test value'

            start_response('200 OK', [('Content-type', 'text/plain')])
            yield ("test_key is: %s\n" % cm.get_cache('test')['test_key']).encode('utf-8')
            cm.get_cache('test').clear()

            try:
                test_value = cm.get_cache('test')['test_key']
            except KeyError:
                yield "test_key cleared".encode('utf-8')
            else:
                yield (
                    "test_key wasn't cleared, is: %s\n" % cm.get_cache('test')['test_key']
                ).encode('utf-8')

        cls.simple_session_app = staticmethod(simple_session_app)
        cls.simple_app = staticmethod(simple_app)
        cls.using_none_app = staticmethod(using_none_app)
        cls.cache_manager_app = staticmethod(cache_manager_app)

    def setUp(self):
        Cache('test', **self.CACHE_ARGS).clear()

    def test_session(self):
        app = WebTestApp(SessionMiddleware(self.simple_session_app, **self.CACHE_ARGS))
        res = app.get('/')
        assert 'current value is: 1' in res
        res = app.get('/')
        assert 'current value is: 2' in res
        res = app.get('/')
        assert 'current value is: 3' in res

    def test_session_invalid(self):
        app = WebTestApp(SessionMiddleware(self.simple_session_app, **self.CACHE_ARGS))
        res = app.get('/invalid', headers=dict(
            Cookie='beaker.session.id=df7324911e246b70b5781c3c58328442; Path=/'))
        assert 'current value is: 2' in res

    def test_session_timeout(self):
        app = WebTestApp(SessionMiddleware(self.simple_session_app, timeout=1, **self.CACHE_ARGS))

        session = app.app._get_session()
        session.save()
        if self.SUPPORTS_TIMEOUT:
            assert session.namespace.timeout == 121

        res = app.get('/')
        assert 'current value is: 1' in res
        res = app.get('/')
        assert 'current value is: 2' in res
        res = app.get('/')
        assert 'current value is: 3' in res

    def test_has_key(self):
        cache = Cache('test', **self.CACHE_ARGS)
        o = object()
        cache.set_value("test", o)
        assert cache.has_key("test")
        assert "test" in cache
        assert not cache.has_key("foo")
        assert "foo" not in cache
        cache.remove_value("test")
        assert not cache.has_key("test")

    def test_clear(self):
        cache = Cache('test', **self.CACHE_ARGS)
        cache.set_value('test', 20)
        cache.set_value('fred', 10)
        assert cache.has_key('test')
        assert 'test' in cache
        assert cache.has_key('fred')
        cache.clear()
        assert not cache.has_key("test")

    def test_has_key_multicache(self):
        cache = Cache('test', **self.CACHE_ARGS)
        o = object()
        cache.set_value("test", o)
        assert cache.has_key("test")
        assert "test" in cache
        cache = Cache('test', **self.CACHE_ARGS)
        assert cache.has_key("test")

    def test_unicode_keys(self):
        cache = Cache('test', **self.CACHE_ARGS)
        o = object()
        cache.set_value(u_('hiŏ'), o)
        assert u_('hiŏ') in cache
        assert u_('hŏa') not in cache
        cache.remove_value(u_('hiŏ'))
        assert u_('hiŏ') not in cache

    def test_long_unicode_keys(self):
        cache = Cache('test', **self.CACHE_ARGS)
        o = object()
        long_str = u_(
            'Очень длинная строка, которая не влезает в сто двадцать восемь байт и поэтому не проходит ограничение в check_key, что очень прискорбно, не правда ли, друзья? Давайте же скорее исправим это досадное недоразумение!'
        )
        cache.set_value(long_str, o)
        assert long_str in cache
        cache.remove_value(long_str)
        assert long_str not in cache

    def test_spaces_in_unicode_keys(self):
        cache = Cache('test', **self.CACHE_ARGS)
        o = object()
        cache.set_value(u_('hi ŏ'), o)
        assert u_('hi ŏ') in cache
        assert u_('hŏa') not in cache
        cache.remove_value(u_('hi ŏ'))
        assert u_('hi ŏ') not in cache

    def test_spaces_in_keys(self):
        cache = Cache('test', **self.CACHE_ARGS)
        cache.set_value("has space", 24)
        assert cache.has_key("has space")
        assert 24 == cache.get_value("has space")
        cache.set_value("hasspace", 42)
        assert cache.has_key("hasspace")
        assert 42 == cache.get_value("hasspace")

    def test_increment(self):
        app = WebTestApp(CacheMiddleware(self.simple_app))
        res = app.get('/', extra_environ={'beaker.clear': True})
        assert 'current value is: 1' in res
        res = app.get('/')
        assert 'current value is: 2' in res
        res = app.get('/')
        assert 'current value is: 3' in res

        app = WebTestApp(CacheMiddleware(self.simple_app))
        res = app.get('/', extra_environ={'beaker.clear': True})
        assert 'current value is: 1' in res
        res = app.get('/')
        assert 'current value is: 2' in res
        res = app.get('/')
        assert 'current value is: 3' in res

    def test_cache_manager(self):
        app = WebTestApp(CacheMiddleware(self.cache_manager_app))
        res = app.get('/')
        assert 'test_key is: test value' in res
        assert 'test_key cleared' in res

    def test_store_none(self):
        app = WebTestApp(CacheMiddleware(self.using_none_app))
        res = app.get('/', extra_environ={'beaker.clear': True})
        assert 'current value is: 10' in res
        res = app.get('/')
        assert 'current value is: None' in res

    def test_expiretime(self):
        cache = Cache('test', **self.CACHE_ARGS)
        cache.set_value("has space", 24, expiretime=1)
        assert cache.has_key("has space")
        time.sleep(1.1)
        assert not cache.has_key("has space")

    def test_expiretime_automatic(self):
        if not self.SUPPORTS_EXPIRATION:
            self.skipTest('NamespaceManager does not support automatic expiration')

        cache = Cache('test', **self.CACHE_ARGS)
        cache.set_value("has space", 24, expiretime=1)
        assert cache.namespace.has_key("has space")
        time.sleep(1.1)
        assert not cache.namespace.has_key("has space")

    def test_createfunc(self):
        cache = Cache('test', **self.CACHE_ARGS)

        def createfunc():
            createfunc.count += 1
            return createfunc.count
        createfunc.count = 0

        def keepitlocked():
            lock = cache.namespace.get_creation_lock('test')
            lock.acquire()
            keepitlocked.acquired = True
            time.sleep(1.0)
            lock.release()
        keepitlocked.acquired = False

        v0 = cache.get_value('test', createfunc=createfunc)
        self.assertEqual(v0, 1)

        v0 = cache.get_value('test', createfunc=createfunc)
        self.assertEqual(v0, 1)

        cache.remove_value('test')

        begin = datetime.datetime.utcnow()
        t = threading.Thread(target=keepitlocked)
        t.start()
        while not keepitlocked.acquired:
            # Wait for the thread that should lock the cache to start.
            time.sleep(0.001)

        v0 = cache.get_value('test', createfunc=createfunc)
        self.assertEqual(v0, 2)

        # Ensure that the `get_value` was blocked by the concurrent thread.
        assert datetime.datetime.utcnow() - begin > datetime.timedelta(seconds=1)

        t.join()



import os
import sys
import re
import inspect

from setuptools import setup, find_packages

py_version = sys.version_info[:2]
here = os.path.abspath(os.path.dirname(__file__))
v = open(os.path.join(here, 'beaker', '__init__.py'))
VERSION = re.compile(r".*__version__ = '(.*?)'", re.S).match(v.read()).group(1)
v.close()

try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''


INSTALL_REQUIRES = []
if not hasattr(inspect, 'signature'):
    # On Python 2.6, 2.7 and 3.2 we need funcsigs dependency
    INSTALL_REQUIRES.append('funcsigs')


TESTS_REQUIRE = ['pytest', 'pycryptodome']

if py_version == (2, 6):
    TESTS_REQUIRE.append('WebTest<2.0.24')
    TESTS_REQUIRE.append('pycparser==2.18')
else:
    TESTS_REQUIRE.append('webtest')

if py_version == (3, 2):
    TESTS_REQUIRE.append('coverage < 4.0')
else:
    TESTS_REQUIRE.append('coverage')

if py_version == (3, 3):
    TESTS_REQUIRE.append('cryptography < 2.1.0')
else:
    TESTS_REQUIRE.append('cryptography')

if not sys.platform.startswith('java') and not sys.platform == 'cli':
    if py_version == (2, 6):
        TESTS_REQUIRE.append('sqlalchemy < 1.2')
    else:
        TESTS_REQUIRE.append('sqlalchemy')
    TESTS_REQUIRE.extend(['pymongo', 'redis'])
    try:
        import sqlite3
    except ImportError:
        TESTS_REQUIRE.append('pysqlite')
    TESTS_REQUIRE.extend(['pylibmc', 'python-memcached'])


setup(name='Beaker',
      version=VERSION,
      description="A Session and Caching library with WSGI Middleware",
      long_description=README,
      classifiers=[
      'Development Status :: 5 - Production/Stable',
      'Environment :: Web Environment',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: BSD License',
      'Programming Language :: Python',
      'Programming Language :: Python :: 3.7',
      'Programming Language :: Python :: 3.8',
      'Programming Language :: Python :: 3.9',
      'Programming Language :: Python :: 3.10',
      'Programming Language :: Python :: 3.11',
      'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      'Topic :: Internet :: WWW/HTTP :: WSGI',
      'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
      ],
      keywords='wsgi myghty session web cache middleware',
      author='Ben Bangert, Mike Bayer, Philip Jenvey, Alessandro Molina',
      author_email='ben@groovie.org, pjenvey@groovie.org, amol@turbogears.org',
      url='https://beaker.readthedocs.io/',
      license='BSD',
      license_files=['LICENSE'],
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests', 'tests.*']),
      zip_safe=False,
      install_requires=INSTALL_REQUIRES,
      extras_require={
          'crypto': ['pycryptopp>=0.5.12'],
          'pycrypto': ['pycrypto'],
          'pycryptodome': ['pycryptodome'],
          'cryptography': ['cryptography'],
          'testsuite': [TESTS_REQUIRE]
      },
      test_suite='tests',
      tests_require=TESTS_REQUIRE,
      entry_points="""
          [paste.filter_factory]
          beaker_session = beaker.middleware:session_filter_factory

          [paste.filter_app_factory]
          beaker_session = beaker.middleware:session_filter_app_factory

          [beaker.backends]
          database = beaker.ext.database:DatabaseNamespaceManager
          memcached = beaker.ext.memcached:MemcachedNamespaceManager
          google = beaker.ext.google:GoogleNamespaceManager
          sqla = beaker.ext.sqla:SqlaNamespaceManager
      """
)



from beaker.middleware import SessionMiddleware
from unittest import SkipTest
try:
    from webtest import TestApp as WebTestApp
except ImportError:
    raise SkipTest("webtest not installed")

def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)

def simple_app(environ, start_response):
    session = environ['beaker.session']
    domain = environ.get('domain')
    if domain:
        session.domain = domain
    if 'value' not in session:
        session['value'] = 0
    session['value'] += 1
    if not environ['PATH_INFO'].startswith('/nosave'):
        session.save()
    start_response('200 OK', [('Content-type', 'text/plain')])
    msg = 'The current value is: %s, session id is %s' % (session.get('value', 0),
                                                          session.id)
    return [msg.encode('utf-8')]


def test_same_domain():
    options = {'session.data_dir':'./cache',
               'session.secret':'blah',
               'session.cookie_domain': '.hoop.com'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/', extra_environ=dict(HTTP_HOST='subdomain.hoop.com'))
    assert 'current value is: 1' in res
    assert 'Domain=.hoop.com' in res.headers['Set-Cookie']
    res = app.get('/', extra_environ=dict(HTTP_HOST='another.hoop.com'))
    assert 'current value is: 2' in res
    assert [] == res.headers.getall('Set-Cookie')
    res = app.get('/', extra_environ=dict(HTTP_HOST='more.subdomain.hoop.com'))
    assert 'current value is: 3' in res


def test_different_domain():
    options = {'session.data_dir':'./cache',
               'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/', extra_environ=dict(domain='.hoop.com',
                                          HTTP_HOST='www.hoop.com'))
    res = app.get('/', extra_environ=dict(domain='.hoop.co.uk',
                                          HTTP_HOST='www.hoop.com'))
    assert 'Domain=.hoop.co.uk' in res.headers['Set-Cookie']
    assert 'current value is: 2' in res

    res = app.get('/', extra_environ=dict(domain='.hoop.co.uk',
                                          HTTP_HOST='www.test.com'))
    assert 'current value is: 1' in res


if __name__ == '__main__':
    from paste import httpserver
    wsgi_app = SessionMiddleware(simple_app, {})
    httpserver.serve(wsgi_app, host='127.0.0.1', port=8080)



from beaker.cache import Cache
from . import base


class TestMongoDB(base.CacheManagerBaseTests):
    SUPPORTS_TIMEOUT = False
    CACHE_ARGS = {
        'type': 'ext:mongodb',
        'url': 'mongodb://localhost:27017/beaker_testdb'
    }

    def test_client_reuse(self):
        cache1 = Cache('test1', **self.CACHE_ARGS)
        cli1 = cache1.namespace.client
        cache2 = Cache('test2', **self.CACHE_ARGS)
        cli2 = cache2.namespace.client
        self.assertTrue(cli1 is cli2)


import re
import unittest

from beaker.middleware import SessionMiddleware
try:
    from webtest import TestApp as WebTestApp
except ImportError:
    raise unittest.SkipTest("webtest not installed")


def teardown_module():
    import shutil
    shutil.rmtree('./cache', True)

def no_save_app(environ, start_response):
    session = environ['beaker.session']
    sess_id = environ.get('SESSION_ID')
    start_response('200 OK', [('Content-type', 'text/plain')])
    msg = 'The current value is: %s, session id is %s' % (session.get('value'),
                                                          session.id)
    return [msg.encode('utf-8')]

def simple_app(environ, start_response):
    session = environ['beaker.session']
    sess_id = environ.get('SESSION_ID')
    if sess_id:
        session = session.get_by_id(sess_id)
    if not session:
        start_response('200 OK', [('Content-type', 'text/plain')])
        return [("No session id of %s found." % sess_id).encode('utf-8')]
    if not 'value' in session:
        session['value'] = 0
    session['value'] += 1
    if not environ['PATH_INFO'].startswith('/nosave'):
        session.save()
    start_response('200 OK', [('Content-type', 'text/plain')])
    msg = 'The current value is: %s, session id is %s' % (session.get('value'),
                                                          session.id)
    return [msg.encode('utf-8')]

def simple_auto_app(environ, start_response):
    """Like the simple_app, but assume that sessions auto-save"""
    session = environ['beaker.session']
    sess_id = environ.get('SESSION_ID')
    if sess_id:
        session = session.get_by_id(sess_id)
    if not session:
        start_response('200 OK', [('Content-type', 'text/plain')])
        return [("No session id of %s found." % sess_id).encode('utf-8')]
    if not 'value' in session:
        session['value'] = 0
    session['value'] += 1
    if environ['PATH_INFO'].startswith('/nosave'):
        session.revert()
    start_response('200 OK', [('Content-type', 'text/plain')])
    msg = 'The current value is: %s, session id is %s' % (session.get('value', 0),
                                                          session.id)
    return [msg.encode('utf-8')]

def test_no_save():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(no_save_app, **options))
    res = app.get('/')
    assert 'current value is: None' in res
    assert [] == res.headers.getall('Set-Cookie')


def test_increment():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res

def test_increment_auto():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_auto_app, auto=True, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res
    res = app.get('/')
    assert 'current value is: 3' in res


def test_different_sessions():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    app2 = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    res = app2.get('/')
    res = app2.get('/')
    res2 = app.get('/')
    assert 'current value is: 2' in res2
    assert 'current value is: 4' in res

def test_different_sessions_auto():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_auto_app, auto=True, **options))
    app2 = WebTestApp(SessionMiddleware(simple_auto_app, auto=True, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    assert 'current value is: 1' in res
    res = app2.get('/')
    res = app2.get('/')
    res = app2.get('/')
    res2 = app.get('/')
    assert 'current value is: 2' in res2
    assert 'current value is: 4' in res

def test_nosave():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/nosave')
    assert 'current value is: 1' in res
    res = app.get('/nosave')
    assert 'current value is: 1' in res

    res = app.get('/')
    assert 'current value is: 1' in res
    res = app.get('/')
    assert 'current value is: 2' in res

def test_revert():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_auto_app, auto=True, **options))
    res = app.get('/nosave')
    assert 'current value is: 0' in res
    res = app.get('/nosave')
    assert 'current value is: 0' in res

    res = app.get('/')
    assert 'current value is: 1' in res
    assert [] == res.headers.getall('Set-Cookie')
    res = app.get('/')
    assert [] == res.headers.getall('Set-Cookie')
    assert 'current value is: 2' in res

    # Finally, ensure that reverting shows the proper one
    res = app.get('/nosave')
    assert [] == res.headers.getall('Set-Cookie')
    assert 'current value is: 2' in res

def test_load_session_by_id():
    options = {'session.data_dir':'./cache', 'session.secret':'blah'}
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res
    res = app.get('/')
    res = app.get('/')
    assert 'current value is: 3' in res
    old_id = re.sub(r'^.*?session id is (\S+)$', r'\1', res.body.decode('utf-8'), re.M)

    # Clear the cookies and do a new request
    app = WebTestApp(SessionMiddleware(simple_app, **options))
    res = app.get('/')
    assert 'current value is: 1' in res

    # Load a bogus session to see that its not there
    res = app.get('/', extra_environ={'SESSION_ID': 'jil2j34il2j34ilj23'})
    assert 'No session id of' in res

    # Saved session was at 3, now it'll be 4
    res = app.get('/', extra_environ={'SESSION_ID': str(old_id)})
    assert 'current value is: 4' in res

    # Prior request is now up to 2
    res = app.get('/')
    assert 'current value is: 2' in res


if __name__ == '__main__':
    from paste import httpserver
    wsgi_app = SessionMiddleware(simple_app, {})
    httpserver.serve(wsgi_app, host='127.0.0.1', port=8080)



# -*- coding: utf-8 -*-
"""This is a collection of annotated functions used by tests.

They are grouped here to provide an easy way to import them at runtime
to check whenever tests for annotated functions should be skipped or not
on current python version.
"""
from beaker.cache import cache_region
import time

class AnnotatedAlfredCacher(object):
    @cache_region('short_term')
    def alfred_self(self, xx: int, y=None) -> str:
        return str(time.time()) + str(self) + str(xx) + str(y)

